<?php
/**
 * Amazon Helper module
 * http://www.aa-team.com
 * ======================
 *
 * @package		WooZone Contextual
 * @author		Andrei Dinca, AA-Team
 * @version		1.0
 */
!defined('ABSPATH') and exit;
if (class_exists('WZC_Amazon_Helper') != true) {
    class WZC_Amazon_Helper
    {
        
        const VERSION = '1.0';
        
		// amazon webservice object
		public $aaAmazonWS = null;
		
		public $country_data = array();
        /**
         * parent storage
         *
         * @var array
         */
        public $config = array(
			"protocol" 						=> "xml", 
			"country" 						=> "com",
			"frontend_show_free_shipping" 	=> "yes",
			"frontend_show_coupon_text" 	=> "yes",
			"link_color" 					=> "#E47911",
			"link_target"                   => "no",
			"checkout_type" 				=> "self",
			"90day_cookie" 					=> "yes",
			"price_setup" 					=> "only_amazon",
			"services_used_forip" 			=> "local_csv",
			"show_short_description" 		=> "yes",
			"main_aff_id" 					=> "com",
			"AffiliateID"					=> array(
				"com"	=> '',
				"uk"	=> '',
				"de"	=> '',
				"fr"	=> '',
				"jp"	=> '',
				"ca"	=> '',
				"cn"	=> '',
				"in"	=> '',
				"it"	=> '',
				"es"	=> '',
				'mx' => '',
				'br' => '',
			),
		);
        
        /**
         * The constructor
         */
        public function __construct()
        {
        	// create AJAX request
			add_action('wp_ajax_WZC_launch_amz_search', array(
                $this,
                'amazon_search'
            ));
			
			$this->country_data = array(
               	'com' => array( 'cc' => 'us', 'mplace' => 'US'),
				'uk' => array( 'cc' => 'uk', 'mplace' => 'GB'),
               	'de' => array( 'cc' => 'de', 'mplace' => 'DE'),
               	'fr' => array( 'cc' => 'fr', 'mplace' => 'FR'),
               	'jp' => array( 'cc' => 'jp', 'mplace' => 'JP'),
               	'ca' => array( 'cc' => 'ca', 'mplace' => 'CA'),
               	'cn' => array( 'cc' => 'cn', 'mplace' => 'CN'),
               	'in' => array( 'cc' => 'in', 'mplace' => 'IN'),
               	'it' => array( 'cc' => 'it', 'mplace' => 'IT'),
               	'es' => array( 'cc' => 'es', 'mplace' => 'ES'),
               	'mx' => array( 'cc' => 'mx', 'mplace' => 'MX'),
               	'br' => array( 'cc' => 'br', 'mplace' => 'BR'),
			);
			
        	$this->setup();
        }

		private function setup()
		{
			$db_settings = get_option( 'WZC_Amazon_Settings', true );
			if( count($db_settings) > 0 && $db_settings !== true ){
				$this->config = array_merge( $this->config, $db_settings );
			}
		}

		public function get_region()
		{
			if( isset($this->country_data[ WZC_country2key( $this->config['country'] ) ]) ){
				return $this->country_data[ WZC_country2key( $this->config['country'] ) ]['mplace'];
			}
			
			return 'US';
		}
		
		public function amazon_setup()
		{
			// load the amazon webservices client class
			require_once WZC_path_dir( 'HELPERS_DIR', 'aaAmazonWS.class.php' );

			// create new amazon instance
			$this->aaAmazonWS = new aaAmazonWS(
				trim($this->config['AccessKeyID']),
				trim($this->config['SecretAccessKey']),
				$this->config['country'],
				$this->main_aff_id()
			); 
		}
		
		public function amazon_search()
		{
			$html = array();
			
			// init the amazon transporter
			$this->amazon_setup();
			
			$params = array(
				'search_by' => isset( $_REQUEST['search_by'] ) ? $_REQUEST['search_by'] : '',
				'keyword' 	=> isset( $_REQUEST['keyword'] ) ? trim($_REQUEST['keyword']) : '',
				'category' 	=> isset( $_REQUEST['node'] ) ? $_REQUEST['node'] : '',
				'asin' 		=> isset( $_REQUEST['asin'] ) ? $_REQUEST['asin'] : '',
				'paged'		=> isset( $_REQUEST['paged'] ) ? (int) $_REQUEST['paged'] : 1,
				'post_id'	=> isset( $_REQUEST['post_id'] ) ? (int) $_REQUEST['post_id'] : 0
			);
   
			$total_pages = 10;
			
			if( $params['search_by'] == 'find-products-into-content' ){
				try {
					$products = array();

					$cache_name = 'wzc_amz_' . md5($params['asin']);
   
					// try to get from cache 
					$products = get_transient( $cache_name ); 
					if( $products != false && is_array($products) && count($products) > 0 ){
						die( WooZone_Contextual()->autofind->find_products_into_content( $products, $params['post_id'] ) );
					}
					
                    //Large,ItemAttributes,OfferFull,Offers,Variations,Reviews,PromotionSummary,SalesRank
		        	$this->aaAmazonWS->responseGroup('Large,ItemAttributes,OfferFull,Offers,Variations,PromotionSummary');
					
					$response = $this->aaAmazonWS->lookup( $params['asin'] );
                    //var_dump('<pre>', $response, '</pre>'); die('debug...'); 
					
                    $respStatus = $this->is_amazon_valid_response( $response );
                    if ( $respStatus['status'] == 'valid' ) {
						
                        // verify array of Items or array of Item elements
                        if ( isset($response['Items']['Item']['ASIN']) ) {
                            $response['Items']['Item'] = array( $response['Items']['Item'] );
                        }
						foreach ( $response['Items']['Item'] as $item ){
							// speed up php by cleaning unused keys
							if( isset($item['ItemLinks'])) unset($item['ItemLinks']);
							if( isset($item['MediumImage'])) unset($item['MediumImage']);
							//if( isset($item['ImageSets'])) unset($item['ImageSets']);
							if( isset($item['CustomerReviews'])) unset($item['CustomerReviews']);
							if( isset($item['SimilarProducts'])) unset($item['SimilarProducts']);
							if( isset($item['BrowseNodes'])) unset($item['BrowseNodes']);
							//if( isset($item['Offers'])) unset($item['Offers']);
							
							$products[] = $this->product_item( $item );
						}
 
						if( count($products) > 0 ){
							// basic cache sistem
							set_transient( $cache_name, $products, 1000 );
							
							die( WooZone_Contextual()->autofind->find_products_into_content( $products, $params['post_id'] ) );
						}
					} else { // INVALID
					    $html[] = $respStatus['html'];
					}
					
				} catch (Exception $e) {
		            // Check 
		            if (isset($e->faultcode)) {
		                //var_dump('<pre>',$e,'</pre>'); die;
						$__msg = $e->faultcode .  ' : ' . (isset($e->faultstring) ? $e->faultstring : $e->getMessage());
		            }
					$__msg = $e->getMessage();
					$html[] = $__msg;
		        }
			}

			else if( $params['search_by'] == 'asin' ){
				try {
					$this->aaAmazonWS->category( 'All' );
					
					if( $this->config['price_setup'] == 'only_amazon' ){
						$this->aaAmazonWS->optionalParameters(array('MerchantId' => 'Amazon'));
					}
					
					//Large,ItemAttributes,OfferFull,Offers,Variations,Reviews,PromotionSummary,SalesRank
		        	$this->aaAmazonWS->responseGroup('Large,ItemAttributes,OfferFull,Offers,Variations,PromotionSummary');
		         
					$response = $this->aaAmazonWS->lookup( $params['asin'] );
					//var_dump('<pre>',$response,'</pre>'); die; // debug
					  
                    $respStatus = $this->is_amazon_valid_response( $response );
                    if ( $respStatus['status'] == 'valid' ) {

						$item = $response['Items']['Item'];
						// speed up php by cleaning unused keys
						if( isset($item['ItemLinks'])) unset($item['ItemLinks']);
						if( isset($item['MediumImage'])) unset($item['MediumImage']);
						//if( isset($item['ImageSets'])) unset($item['ImageSets']);
						if( isset($item['CustomerReviews'])) unset($item['CustomerReviews']);
						if( isset($item['SimilarProducts'])) unset($item['SimilarProducts']);
						if( isset($item['BrowseNodes'])) unset($item['BrowseNodes']);
						//if( isset($item['Offers'])) unset($item['Offers']);
						
						$product = $this->product_item( $item );
						
						die( json_encode(
							array(
								'status' 	=> 'valid',
								'product'	=> $product
							)
						) );
					} else { // INVALID
					    $html[] = $respStatus['html'];
					}
					
				} catch (Exception $e) {
		            // Check 
		            if (isset($e->faultcode)) {
		                //var_dump('<pre>',$e,'</pre>'); die;
						$__msg = $e->faultcode .  ' : ' . (isset($e->faultstring) ? $e->faultstring : $e->getMessage());
		            }
					$__msg = $e->getMessage();
					$html[] = $__msg;
		        }
			}

			else if( $params['search_by'] == 'keyword' ){

				try {
					if( $params['category'] == 'All Departments' ){
						$this->aaAmazonWS->category( 'All' );
						$total_pages = 5;
					}
					else{
						$this->aaAmazonWS->category( $params['category'] );
						$total_pages = 10;
					}
					
					if( $this->config['price_setup'] == 'only_amazon' ){
						$this->aaAmazonWS->optionalParameters(array('MerchantId' => 'Amazon'));
					}
					
					if( $params['paged'] > 1 ){
						$this->aaAmazonWS->page( $params['paged'] );
					}

					//Large,ItemAttributes,OfferFull,Offers,Variations,Reviews,PromotionSummary,SalesRank
		        	$this->aaAmazonWS->responseGroup('Large,ItemAttributes,OfferFull,Offers,Variations,PromotionSummary');
		         
					$response = $this->aaAmazonWS->search( $params['keyword'] );
					//var_dump('<pre>',$response,'</pre>'); die; // debug
					
					$respStatus = $this->is_amazon_valid_response( $response );
					if ( $respStatus['status'] == 'valid' ) {
					    
						$html[] = '<div class="WZC-item-list-wrapper WZC-item-searched" style="display: block;">';
						$html[] = 	'<ul class="WZC-item-list">';

						//if ( isset($response['Items']['TotalResults']) ) {
						//	$totalItems = isset($response['Items']['TotalResults']) ? $response['Items']['TotalResults'] : 0;
						//	$totalPages = $totalItems > 0 ? ceil( $totalItems / $total_pages ) : 0;
						//}

                        // verify array of Items or array of Item elements
                        if ( isset($response['Items']['Item']['ASIN']) ) {
                            $response['Items']['Item'] = array( $response['Items']['Item'] );
                        }
						
						foreach ( $response['Items']['Item'] as $item ){
							
							//:: speed up php by cleaning unused keys
							if( isset($item['ItemLinks'])) unset($item['ItemLinks']);
							if( isset($item['MediumImage'])) unset($item['MediumImage']);
							//if( isset($item['ImageSets'])) unset($item['ImageSets']);
							if( isset($item['CustomerReviews'])) unset($item['CustomerReviews']);
							if( isset($item['SimilarProducts'])) unset($item['SimilarProducts']);
							if( isset($item['BrowseNodes'])) unset($item['BrowseNodes']);
							//if( isset($item['Offers'])) unset($item['Offers']);
							
							$product = $this->product_item( $item );
							
							$html[] = '<li>';
							$html[] = 	'<textarea name="WZC-prod-json" style="display:none">' . ( json_encode( $product ) ) . '</textarea>';
							$html[] = 	'<div class="WZC-prod-img">';
							$html[] = 		'<img src="' . ( $product['SmallImage'] ) . '">';
							$html[] = 	'</div>';
							$html[] = 	'<div class="WZC-prod-desc">';
							$html[] = 		'<div class="WZC-prod-title"><a href="' . ( $product['DetailPageURL'] ) . '" target="_blank">' . ( $product['Title'] ) . '</a></div>';
							$html[] = 		'<div class="WZC-prod-by"> by <a href="' . ( $product['DetailPageURL'] ) . '" target="_blank">' . ( $product['Binding'] ) . '</a> </div>';
							$html[] = 		'<div class="WZC-price"><del>' . ( $product['ListPrice'] ) . '</del><span>' . ( $product['OfferSummary'] ) . '</span></div>';
							$html[] = 	'</div>';
							$html[] = 	'<button class="WZC-item-action small-btn WZC-add-btn">Add</button>';
							$html[] = '</li>';
						} // end foreach
						
						$html[] = 	'</ul>';
						
						$prev_page = $params['paged'] - 1;
						if( $prev_page <= 1 ){
							$prev_page = 1;
						}
						$next_page = $params['paged'] + 1;
						if( $next_page >= $total_pages ){
							$next_page = $total_pages;
						}
						$html[] = 	'<div class="WZC-items-pagination">';
						$html[] = 		'<div style="float: right; padding-top: 3px; padding-right: 5px;">';
						$html[] = 			'from';
						$html[] = 			'<span class="start-search-list-no">' . ( $params['paged'] ) . '</span> to';
						$html[] = 			'<span class="end-search-list-no">' . ( $total_pages ) . '</span>';
						$html[] = 		'</div>';
						
						$html[] = 		'<a class="WZC-pagination-close" href="#">Close</a>';
						$html[] = 		'<button class="WZC-pagination-prev small-btn" data-paged="' . ( $prev_page ) . '" href="#">Prev</button>';
						$html[] = 		'<button class="WZC-pagination-next small-btn" data-paged="' . ( $next_page ) . '" href="#">Next</button>';
						$html[] = 	'</div>';
						$html[] = '</div>';
					} else { // INVALID

					    //$html[] = '<div class="WZC-item-list-wrapper WZC-item-searched" style="display: block;">';
                        $html[] = $respStatus['html'];
                        //$html[] = '</div>';
					}
					
				} catch (Exception $e) {
		            // Check 
		            if (isset($e->faultcode)) {
		                //var_dump('<pre>',$e,'</pre>'); die;
						$__msg = $e->faultcode .  ' : ' . (isset($e->faultstring) ? $e->faultstring : $e->getMessage());
		            }
					$__msg = $e->getMessage();
					$html[] = $__msg;
		        }
			}
			
			die( json_encode(
				array(
					'status' 	=> isset($respStatus['status']) ? $respStatus['status'] : 'invalid',
					'html'		=> implode( "\n", $html )
				)
			) );   
		}

        public function is_amazon_valid_response( $response ) {
            $ret = array(
                'status'        => 'invalid',
                'html'          => 'unknown message.',
                'code'          => -1,
            );

            // parse amazon response
            if ( !isset($response['Items']['Request']['IsValid']) ) {

                $msg = 'invalid amazon response.';
                return array_merge($ret, array(
                    'html'      => '<div class="error" style="margin: 0px; padding: 6px;">' . $msg . '</div>',
                    'code'      => 1,
                ));
            }
  
            if ( $response['Items']['Request']['IsValid'] == 'False' ) {
        
                if ( isset($response['Items']['Request']['Errors']['Error']['Code']) ) {
                    $msg = 'Amazon error id: <bold>' . ( $response['Items']['Request']['Errors']['Error']['Code'] ) . '</bold>: <br /> ' . ( $response['Items']['Request']['Errors']['Error']['Message'] );
                } else {
                    $msg = 'unknown amazon error.';
                }
                return array_merge($ret, array(
                    'html'      => '<div class="error" style="margin: 0px; padding: 6px;">' . $msg . '</div>',
                    'code'      => 2, 
                ));
            }
            
            // No products found!
            //isset($response['Items']['Item']) && count($response['Items']['Item']) > 0
            if ( ( count($response['Items']) <= 0 )
                || !isset($response['Items']['Item'])
                || ( count($response['Items']['Item']) <= 0 ) ) {

                if ( isset($response['Items']['Request']['Errors']['Error']['Code']) ) {
                    $msg = 'Amazon error id: <bold>' . ( $response['Items']['Request']['Errors']['Error']['Code'] ) . '</bold>: <br /> ' . ( $response['Items']['Request']['Errors']['Error']['Message'] );
                    if ( 'AWS.ECommerceService.NoExactMatches' == $response['Items']['Request']['Errors']['Error']['Code'] ) {
                        $msg = 'Sorry, your search did not return any results';
                    }
                } else {
                    $msg = 'no products found.';
                }
                return array_merge($ret, array(
                    'html'      => '<div class="error" style="margin: 0px; padding: 6px;">' . $msg . '</div>',
                    'code'      => 3,
                ));
            }
             
            // success   
            return array_merge($ret, array(
                'status'        => 'valid',
                'html'          => 'valid message',
                'code'          => 0,
            ));
        }

		private function product_item( $item=array() )
		{
			
			// summarize product details
			$ret = array(
				'Title' 			=> isset($item['ItemAttributes']['Title']) ? stripslashes($item['ItemAttributes']['Title']) : '',
				'Brand' 			=> isset($item['ItemAttributes']['Brand']) ? $item['ItemAttributes']['Brand'] : '',
				'ASIN' 				=> isset($item['ASIN']) ? $item['ASIN'] : '',
				'DetailPageURL' 	=> isset($item['DetailPageURL']) ? $item['DetailPageURL'] : '',
				'SmallImage' 		=> isset($item['SmallImage']['URL']) ? trim( $item['SmallImage']['URL'] ) : '',
				'LargeImage' 		=> isset($item['LargeImage']['URL']) ? $item['LargeImage']['URL'] : '',
				'Binding' 			=> isset($item['ItemAttributes']['Binding']) ? $item['ItemAttributes']['Binding'] : '',
				'ListPrice' 		=> isset($item['ItemAttributes']['ListPrice']['FormattedPrice']) ? $item['ItemAttributes']['ListPrice']['FormattedPrice'] : '',
				'OfferSummary' 		=> isset($item['OfferSummary']['LowestNewPrice']['FormattedPrice']) ? $item['OfferSummary']['LowestNewPrice']['FormattedPrice'] : '',
				//'EditorialReviews' 	=> isset($item['EditorialReviews']['EditorialReview'][0]['Content']) ? $item['EditorialReviews']['EditorialReview'][0]['Content'] : ''
			);

            //:: thumbnail
            if ( empty($ret['SmallImage']) ) {
				$opStat = $this->build_images_data( $item );
				$ret['SmallImage'] = $opStat['SmallImage'];
            }
			if ( empty($ret['SmallImage']) ) {
				$opStat = $this->get_first_variation_image( $item );
				$ret['SmallImage'] = $opStat['SmallImage'];
			}

			//:: price
			$price = $this->get_productPrice( $item );
			$ret['ListPrice'] = $price['_regular_price'];
			$ret['OfferSummary'] = $price['_sale_price'];
			
			if ( '' != $ret['ListPrice'] ) {
				$ret['ListPrice'] = $price['_currency'] . ' ' . $ret['ListPrice'];
			}
			if ( '' != $ret['OfferSummary'] ) {
				$ret['OfferSummary'] = $price['_currency'] . ' ' . $ret['OfferSummary'];
			}

            return $ret;
		}

		private function build_images_data( $item=array() ) {
			$ret = array('SmallImage' => '');

			if ( isset($item['ImageSets'], $item['ImageSets']['ImageSet']) && count($item['ImageSets']["ImageSet"]) > 0 ) {
                	
					 // hack if have only 1 item
					if( isset($item['ImageSets']['ImageSet']['SwatchImage']) ){
                    	$_tmp = $item['ImageSets']["ImageSet"];
                    	$item['ImageSets']["ImageSet"] = array();
                    	$item['ImageSets']["ImageSet"][0] = $_tmp;
					}

                    foreach ($item['ImageSets']['ImageSet'] as $key => $value) {

                        if( isset($value['SmallImage']['URL']) ){
                            $ret['SmallImage'] = $value['SmallImage']['URL'];
                            break;
                        }
                    }
			}
			return $ret;
		}
		
		private function get_first_variation_image( $item ) {
            if ( isset($item['Variations'], $item['Variations']['TotalVariations'], $item['Variations']['Item']) ) {
                $total = (int) $item['Variations']['TotalVariations'];
                
                $variations = array();
                if ($total <= 1 || isset($item['Variations']['Item']['ASIN'])) { // --fix 2015.03.19
                    $variations[] = $item['Variations']['Item'];
                } else {
                    $variations = (array) $item['Variations']['Item'];
                }
 
                // Loop through the variation
                foreach ($variations as $variation_item) {
                    
                    $images = $this->build_images_data( $variation_item );
                    if ( !empty($images['SmallImage']) ) {
                        return $images;
                    }
                } // end foreach
            }
		}
		
		public function categs_as_select( $option_name='' )
		{
			$html = array();
			$html[] = '<select name="' . ( $option_name ) . '">';
			foreach ($this->get_categs() as $key => $value) {
				$html[] = '<option value="' . ( $value ) . '">' . ( $value ) . '</option>';
			}
			$html[] = '</select>';
			
			return implode( "\n", $html );
		}
		
		public function get_categs()
		{
			$categs = array();
			
			// add All for any country
			$categs[0] = 'All Departments';
			
			$csv_file = WZC_path_dir( 'AMAZON_WS_DIR', 'assets/nodes-' . ( WZC_country2key( $this->config['country'] ) ) . '.csv' );
			if( !file_exists($csv_file) ){
				wp_die( "Unable to get the csv file from path:" . $csv_file );
			}
			
			// read csv file
			$file_content = WooZone_Contextual()->wp_filesystem->get_contents( $csv_file );
			if( $file_content === false ){
				$file_content = file_get_contents( $csv_file );
				if( $file_content === false ) {
					wp_die( "Unable to open the csv file from path:" . $csv_file );
				}
			}
			
			if( trim($file_content) != "" ){
				foreach ( explode("\n", $file_content) as $row ) {
					$vals = explode( ",", $row );
					
					$categs[ trim($vals[0]) ] = trim($vals[1]);
				}
			}
			
			return $categs;
		}
		
		public function main_aff_id()
		{
			if( isset($this->config['main_aff_id']) && isset($this->config['AffiliateID'][$this->config['main_aff_id']]) ) {
				return $this->config['AffiliateID'][$this->config['main_aff_id']];
			}
		}
		
		public function get_client_ip()
		{
            $ipaddress = '';
            if ($_SERVER['REMOTE_ADDR'])
                $ipaddress = $_SERVER['REMOTE_ADDR'];
            else if ($_SERVER['HTTP_CLIENT_IP'])
                $ipaddress = $_SERVER['HTTP_CLIENT_IP'];
            else if ($_SERVER['HTTP_X_FORWARDED'])
                $ipaddress = $_SERVER['HTTP_X_FORWARDED'];
            else if ($_SERVER['HTTP_FORWARDED_FOR'])
                $ipaddress = $_SERVER['HTTP_FORWARDED_FOR'];
            else if( $_SERVER['HTTP_FORWARDED'])
                $ipaddress = $_SERVER['HTTP_FORWARDED'];
            else if ($_SERVER['HTTP_X_FORWARDED_FOR'])
                $ipaddress = $_SERVER['HTTP_X_FORWARDED_FOR'];

            return $ipaddress;
        }
		
		public function ip2number( $ip ) {
            $long = ip2long($ip);
            if ($long == -1 || $long === false) {
                return false;
            }
            return sprintf("%u", $long);
        }
		
		public function get_country_perip_external( $return_field='country' )
		{
			if ( isset($_SESSION["WZC_country"]) && !empty($_SESSION["WZC_country"]) ) {
				if ( $return_field == 'country' ) {
					return unserialize($_SESSION["WZC_country"]);
				}
			}
   
            $ip = $this->get_client_ip(); 
			$paths = array(
				//'api.hostip.info'			=> 'http://api.hostip.info/country.php?ip={ipaddress}',
				'www.geoplugin.net'			=> 'http://www.geoplugin.net/json.gp?ip={ipaddress}',
				//'www.telize.com'			=> 'http://www.telize.com/geoip/{ipaddress}',
				'ipinfo.io'					=> 'http://ipinfo.io/{ipaddress}/geo',
			);
			
			$service_used = 'www.geoplugin.net';
			if ( isset($this->config['services_used_forip']) && !empty($this->config['services_used_forip']) ) {
				$service_used = $this->config['services_used_forip'];
			}
   
            $country = '';
            if ( $service_used == 'local_csv' ) { // local csv file with ip lists
                
                // read csv hash (string with ip from list)
                $csv_hash = WooZone_Contextual()->wp_filesystem->get_contents( WZC_path_dir( 'AMAZON_WS_DIR', 'assets/GeoIPCountryWhois-hash.csv' ) );
				$csv_hash = explode(',', $csv_hash);
				
                // read csv full (ip from, ip to, country)
                $csv_full = WooZone_Contextual()->wp_filesystem->get_contents( WZC_path_dir( 'AMAZON_WS_DIR', 'assets/GeoIPCountryWhois-full.csv' ) );
                $csv_full = explode(PHP_EOL, $csv_full);
                
                //var_dump('<pre>',count($csv_hash), count($csv_full),'</pre>');
                //var_dump('<pre>',$csv_hash, $csv_full,'</pre>');

                $ip2number = $this->ip2number( $ip );
                //var_dump('<pre>', $ip, $ip2number, '</pre>');
                
                $ipHashIndex = $this->binary_search($ip2number, $csv_hash, array($this, 'binary_search_cmp'));
                if ( $ipHashIndex < 0 ) { // verify if is between (ip_from, ip_to) of csv row
                    $ipHashIndex = abs( $ipHashIndex );
                    $ipFullRow = $csv_full["$ipHashIndex"];
                    $csv_row = explode(',', $ipFullRow);
                    if ( $ip2number >= $csv_row[0] && $ip2number <= $csv_row[1] ) {
                        $country = $csv_row[2];
                    }
                } else { // exact match in the list as ip_from of csv row
                    $ipFullRow = $csv_full["$ipHashIndex"]; 
                    $country = @end( @explode(',', $ipFullRow) );
                }

                if (empty($country)) {
                    $country = 'NOT-FOUND';
                }
                $country = strtoupper( $country );
                
            } else { // external service
			
    			$service_url = $paths["$service_used"];
    			$service_url = str_replace('{ipaddress}', $ip, $service_url);
    
    			$get_user_location = wp_remote_get( $service_url );
    			if ( isset($get_user_location->errors) ) {
    				$country = 'NOT-FOUND';
    			} else {
    					$country = $get_user_location['body'];
    				
						$country = json_decode($country);
						if ( ! is_object($country) ) $country = 'NOT-FOUND';
						else {
							switch ($service_used) {
								//case 'api.hostip.info':
								//	break;
									
								case 'www.geoplugin.net':
									//$country = json_decode($country);
									$country = isset($country->geoplugin_countryCode) ? strtoupper( (string)$country->geoplugin_countryCode ) : 'NOT-FOUND';
									break;
									
								//case 'www.telize.com':
								//	//$country = json_decode($country);
								//	$country = strtoupper( $country->country_code );
								//	break;
									
								case 'ipinfo.io':
									//$country = json_decode($country);
									$country = isset($country->country) ? strtoupper( (string)$country->country ) : 'NOT-FOUND';
									break;
									
								default:
									break;
							}
						}
    			}
            }
			
			if ( $return_field == 'country' ) {
				$user_country = $this->amzForUser($country);
				$_SESSION['WZC_country'] = serialize($user_country);
				return $user_country;
			}
		}
		
		public function main_aff_site()
		{
			if( isset($this->config['main_aff_id']) && isset($this->config['AffiliateID'][$this->config['main_aff_id']]) ) {

				if( $this->config['main_aff_id'] == 'com' ){
					return '.com';
				}
				elseif( $this->config['main_aff_id'] == 'ca' ){
					return '.ca';
				}
				elseif( $this->config['main_aff_id'] == 'cn' ){
					return '.cn';
				}
				elseif( $this->config['main_aff_id'] == 'de' ){
					return '.de';
				}
				elseif( $this->config['main_aff_id'] == 'in' ){
					return '.in';
				}
				elseif( $this->config['main_aff_id'] == 'it' ){
					return '.it';
				}
				elseif( $this->config['main_aff_id'] == 'es' ){
					return '.es';
				}
				elseif( $this->config['main_aff_id'] == 'fr' ){
					return '.fr';
				}
				elseif( $this->config['main_aff_id'] == 'uk' ){
					return '.co.uk';
				}
				elseif( $this->config['main_aff_id'] == 'jp' ){
					return '.co.jp';
				}
				elseif( $this->config['main_aff_id'] == 'mx' ){
					return '.com.mx';
				}
				elseif( $this->config['main_aff_id'] == 'br' ){
					return '.com.br';
				}
			}

			return '';  
		}
		
		private function amzForUser( $userCountry='US' )
		{
			$affIds = $this->config['AffiliateID'];
			$main_aff_id = $this->main_aff_id();
			$main_aff_site = $this->main_aff_site(); 

			if( $userCountry == 'US' ){
				return array(
					'key'	=> 'com',
					'website' => isset($affIds['com']) && (trim($affIds['com']) != "") ? '.com' : $main_aff_site,
					'affID'	=> isset($affIds['com']) && (trim($affIds['com']) != "") ? $affIds['com'] : $main_aff_id
				);
			}
			 
			elseif( $userCountry == 'CA' ){
				return array(
					'key'	=> 'ca',
					'website' => isset($affIds['ca']) && (trim($affIds['ca']) != "") ? '.ca' : $main_aff_site,
					'affID'	=> isset($affIds['ca']) && (trim($affIds['ca']) != "") ? $affIds['ca'] : $main_aff_id
				);
			}
			
			elseif( $userCountry == 'FR' ){
				return array(
					'key'	=> 'fr',
					'website' => isset($affIds['fr']) && (trim($affIds['fr']) != "") ? '.fr' : $main_aff_site,
					'affID'	=> isset($affIds['fr']) && (trim($affIds['fr']) != "") ? $affIds['fr'] : $main_aff_id
				);
			}
			
			elseif( $userCountry == 'CN' ){
				return array(
					'key'	=> 'cn',
					'website' => isset($affIds['cn']) && (trim($affIds['cn']) != "") ? '.cn' : $main_aff_site,
					'affID'	=> isset($affIds['cn']) && (trim($affIds['cn']) != "") ? $affIds['cn'] : $main_aff_id
				);
			}
			
			elseif( $userCountry == 'DE' ){
				return array(
					'key'	=> 'de',
					'website' => isset($affIds['de']) && (trim($affIds['de']) != "") ? '.de' : $main_aff_site,
					'affID'	=> isset($affIds['de']) && (trim($affIds['de']) != "") ? $affIds['de'] : $main_aff_id
				);
			}

			elseif( $userCountry == 'IN' ){
				return array(
					'key'	=> 'in',
					'website' => isset($affIds['in']) && (trim($affIds['in']) != "") ? '.in' : $main_aff_site,
					'affID'	=> isset($affIds['in']) && (trim($affIds['in']) != "") ? $affIds['in'] : $main_aff_id
				);
			}
			
			elseif( $userCountry == 'IT' ){
				return array(
					'key'	=> 'it',
					'website' => isset($affIds['it']) && (trim($affIds['it']) != "") ? '.it' : $main_aff_site,
					'affID'	=> isset($affIds['it']) && (trim($affIds['it']) != "") ? $affIds['it'] : $main_aff_id
				);
			}
			
			elseif( $userCountry == 'JP' ){
				return array(
					'key'	=> 'jp',
					'website' => isset($affIds['jp']) && (trim($affIds['jp']) != "") ? '.co.jp' : $main_aff_site,
					'affID'	=> isset($affIds['jp']) && (trim($affIds['jp']) != "") ? $affIds['jp'] : $main_aff_id
				);
			}
			
			elseif( $userCountry == 'ES' ){
				return array(
					'key'	=> 'es',
					'website' => isset($affIds['es']) && (trim($affIds['es']) != "") ? '.es' : $main_aff_site,
					'affID'	=> isset($affIds['es']) && (trim($affIds['es']) != "") ? $affIds['es'] : $main_aff_id
				);
			}
			
			elseif( $userCountry == 'MX' ){
				return array(
					'key'	=> 'mx',
					'website' => isset($affIds['mx']) && (trim($affIds['mx']) != "") ? '.com.mx' : $main_aff_site,
					'affID'	=> isset($affIds['mx']) && (trim($affIds['mx']) != "") ? $affIds['mx'] : $main_aff_id
				);
			}
			elseif( $userCountry == 'BR' ){
				return array(
					'key'	=> 'br',
					'website' => isset($affIds['br']) && (trim($affIds['br']) != "") ? '.com.br' : $main_aff_site,
					'affID'	=> isset($affIds['br']) && (trim($affIds['br']) != "") ? $affIds['br'] : $main_aff_id
				);
			}
			else{
				
				$website = $this->config["main_aff_id"];
				if( $this->config["main_aff_id"] == 'uk' ) $website = 'co.uk';
				if( $this->config["main_aff_id"] == 'jp' ) $website = 'co.jp';
				if( $this->config["main_aff_id"] == 'mx' ) $website = 'com.mx';
				if( $this->config["main_aff_id"] == 'br' ) $website = 'com.br';
				
				return array(
					'key'			=> $this->config["main_aff_id"],
					'website' 		=> "." . $website,
					'affID'			=> $main_aff_id
				); 
			}
		}

		public function binary_search($key, array $list, $compare_func) 
		{
            $low = 0; 
            $high = count($list) - 1;
     
            while ($low <= $high) {
                $mid = (int) (($high - $low) / 2) + $low; // could use php ceil function
                $cmp = call_user_func($compare_func, $list[$mid], $key);
     
                if ($cmp < 0) {
                    $low = $mid + 1;
                } else if ($cmp > 0) {
                    $high = $mid - 1;
                } else {
                    return $mid;
                }
            }
            return -($low - 1);
        }
		
        public function binary_search_cmp($a, $b) 
        {
            return ($a < $b) ? -1 : (($a > $b) ? 1 : 0);
        }
		
        public function get_productPrice( $thisProd )
        {
            $multiply_factor =  ($this->config['country'] == 'co.jp') ? 1 : 0.01;
            
            $price_setup = (isset($this->config["price_setup"]) && $this->config["price_setup"] == 'amazon_or_sellers' ? 'amazon_or_sellers' : 'only_amazon');
            //$offers_from = ( $price_setup == 'only_amazon' ? 'Amazon' : 'All' );
            
            $ret = array(
                'status'                => 'valid',
                '_price'                => '',
                '_sale_price'           => '',
                '_regular_price'        => '',
                '_price_update_date'    => '',
                '_currency'					=> '',
            );
 
            // list price
            $offers = array(
                'ListPrice'         				=> isset($thisProd['ItemAttributes']['ListPrice']['Amount']) ? ($thisProd['ItemAttributes']['ListPrice']['Amount'] * $multiply_factor ) : '',
                'LowestNewPrice'   		=> isset($thisProd['Offers']['Offer']['OfferListing']['Price']['Amount']) ? ($thisProd['Offers']['Offer']['OfferListing']['Price']['Amount'] * $multiply_factor) : '',
                'LowestNewSalePrice' 	=> isset($thisProd['Offers']['Offer']['OfferListing']['SalePrice']['Amount']) ? ($thisProd['Offers']['Offer']['OfferListing']['SalePrice']['Amount'] * $multiply_factor) : '',
                'LowestPrice'       			=> isset($thisProd['VariationSummary']['LowestSalePrice']['Amount']) ? ($thisProd['VariationSummary']['LowestSalePrice']['Amount'] * $multiply_factor) : '',
                'Offers'            				=> isset($thisProd['Offers']) ? $thisProd['Offers'] : array()
            );
 
            if( $price_setup == 'amazon_or_sellers' && isset($thisProd['OfferSummary']['LowestNewPrice']['Amount']) ) {
                $offers['LowestNewPrice'] = ($thisProd['OfferSummary']['LowestNewPrice']['Amount'] * $multiply_factor);
            }

            // get current product meta, update the values of prices and update it back
            $product_meta = array('product' => array());

			// regular
            $product_meta['product']['regular_price'] = $offers['ListPrice'];
			$product_meta['product']['currency'] = isset($thisProd['ItemAttributes']['ListPrice']['CurrencyCode']) ? $thisProd['ItemAttributes']['ListPrice']['CurrencyCode'] : '';
    
            // regular/ if we don't have a regular price or lowest new price from offer is greater than current list price
            if( 
                ((float)$offers['ListPrice'] == 0.00)
                || ($offers['LowestNewPrice'] > $offers['ListPrice'])
            ) {
                $product_meta['product']['regular_price'] = $offers['LowestNewPrice'];

				// currency
				$product_meta['product']['currency'] = isset($thisProd['Offers']['Offer']['OfferListing']['Price']['CurrencyCode']) ? $thisProd['Offers']['Offer']['OfferListing']['Price']['CurrencyCode'] : $product_meta['product']['currency'];
				if( $price_setup == 'amazon_or_sellers' && isset($thisProd['OfferSummary']['LowestNewPrice']['Amount']) ) {
					$product_meta['product']['currency'] = isset($thisProd['OfferSummary']['LowestNewPrice']['CurrencyCode']) ? $thisProd['OfferSummary']['LowestNewPrice']['CurrencyCode'] : $product_meta['product']['currency'];
				}
            }
  
            // regular/ if still don't have any regular price, try to get from VariationSummary (ex: Apparel category)
            if( !isset($product_meta['product']['regular_price']) || (float)$product_meta['product']['regular_price'] == 0.00 ) {
                $product_meta['product']['regular_price'] = isset($thisProd['VariationSummary']['LowestPrice']['Amount']) ? ( $thisProd['VariationSummary']['LowestPrice']['Amount'] * $multiply_factor ) : '';

				// currency
				$product_meta['product']['currency'] = isset($thisProd['VariationSummary']['LowestPrice']['CurrencyCode']) ? $thisProd['VariationSummary']['LowestPrice']['CurrencyCode'] : $product_meta['product']['currency'];
            }

			// sale/ from Offers or OfferSummary
            if( isset($offers['LowestNewPrice']) ) {
                $product_meta['product']['sales_price'] = $offers['LowestNewPrice']; 
                // if offer price is higher than regular price, delete the offer
                if( $offers['LowestNewPrice'] >= $product_meta['product']['regular_price'] ){
                    unset($product_meta['product']['sales_price']);
                }
            }
			// sale/ from Offers or OfferSummary - for variation child
			if( $price_setup == 'amazon_or_sellers' || empty($product_meta['product']['sales_price']) ) {
				if( isset($offers['LowestNewSalePrice']) ) {
					$product_meta['product']['sales_price'] = $offers['LowestNewSalePrice']; 
					// if offer price is higher than regular price, delete the offer
					if( $offers['LowestNewSalePrice'] >= $product_meta['product']['regular_price'] ){
						unset($product_meta['product']['sales_price']);
					}
				}
			}

			// sale/ from VariationSummary (ex: Apparel category)
            if( isset($offers['LowestPrice']) && empty($product_meta['product']['sales_price']) ) {
                $product_meta['product']['sales_price'] = $offers['LowestPrice']; 
                // if offer price is higher than regular price, delete the offer
                if( $offers['LowestPrice'] >= $product_meta['product']['regular_price'] ){
                    unset($product_meta['product']['sales_price']);
                }
            }

            // set product price metas!
            $ret['_currency'] = $product_meta['product']['currency'];
            if ( isset($product_meta['product']['sales_price']) && !empty($product_meta['product']['sales_price']) ) {
                $ret['_sale_price'] = $product_meta['product']['sales_price'];
            } else { // new sale price is 0
                $ret['_sale_price'] = '';
            }
            $ret['_price_update_date'] = time();
            $ret['_regular_price'] = $product_meta['product']['regular_price'];
            $ret['_price'] = (isset($product_meta['product']['sales_price']) && trim($product_meta['product']['sales_price']) != ""
            	? $product_meta['product']['sales_price'] : $product_meta['product']['regular_price']);

            return $ret;
        }
	}
}