<?php
/**
 * WZC Main manager.
 *
 * @package WZC
 * @since   1.0
 */
if ( ! function_exists( 'WooZone_Contextual' ) ) {
	/**
	 * WZC manager.
	 * @since 1.0
	 * @return WooZone_Contextual
	 */
	function WooZone_Contextual() {
		global $WooZone_Contextual;

		return $WooZone_Contextual;
	}
}

if ( ! function_exists( 'WZC_Settings' ) ) {
	/**
	 * WZC manager.
	 * @since 1.0
	 * @return WZC_Settings
	 */
	function WZC_Settings() {
		return new WZC_Settings();
	}
}

if ( ! function_exists( 'WZC_CreateTable' ) ) {
	function WZC_CreateTable() {
		global $wpdb;
		
		// check if table exist, if not create table
		$table_name = $wpdb->prefix . "zonecontextual";
		if ($wpdb->get_var( "show tables like '$table_name'" ) != $table_name) {

			$sql = "
				CREATE TABLE IF NOT EXISTS " . $table_name . " (
				  	`id` INT(10) NOT NULL AUTO_INCREMENT,
					`keyword` VARCHAR(100) NOT NULL,
					`words_count` SMALLINT(2) NOT NULL DEFAULT '0',
					`max_replacements` VARCHAR(5) NOT NULL DEFAULT '',
					`product_ASIN` VARCHAR(10) NOT NULL,
					`product_Title` VARCHAR(50) NOT NULL,
					`product_Brand` VARCHAR(50) NOT NULL,
					`product_DetailPageURL` VARCHAR(150) NOT NULL,
					`product_SmallImage` VARCHAR(100) NOT NULL,
					`product_Binding` VARCHAR(50) NOT NULL,
					`product_ListPrice` VARCHAR(50) NOT NULL,
					`product_OfferSummary` VARCHAR(50) NOT NULL,
					`product_EditorialReviews` TEXT NOT NULL,
					`hits` INT(11) NOT NULL DEFAULT '0',
					`redirects` INT(11) NOT NULL DEFAULT '0',
					`date_added` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
					PRIMARY KEY (`id`),
					UNIQUE INDEX `keyword_product_ASIN` (`keyword`, `product_ASIN`)
				);
				";

			require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
			dbDelta($sql);
		}
	}
}
		
		
if ( ! function_exists( 'WZC_Settings_Build_Options' ) ) {
	/**
	 * WZC manager.
	 * @since 1.0
	 * @return WZC_Settings
	 */
	function WZC_Settings_Build_Options( $options=array(), $box_id='' ) {
		return WZC_Settings()->build_options( $options, $box_id );
	}
}

if ( ! function_exists( 'WZC_Amazon_Helper' ) ) {
	/**
	 * WZC manager.
	 * @since 1.0
	 * @return WZC_Settings
	 */
	function WZC_Amazon_Helper() {
		return new WZC_Amazon_Helper();
	}
}

if ( ! function_exists( 'WZC_keywords' ) ) {
	/**
	 * WZC manager.
	 * @since 1.0
	 * @return WZC_Settings
	 */
	function WZC_keywords() {
		return new WZC_keywords();
	}
}


if ( ! function_exists( 'WZC_Load_Options' ) ) {
	/**
	 * WZC manager.
	 * @since 1.0
	 * @return WZC_Settings
	 */
	function WZC_Load_Options( $dir ) {
	    new WZC_Settings(); // save settings first!
	    
		$options = array();
			
		// find if we have a options.php into the same folder
		$options_file = $dir . '/options.php';
		if( is_file($options_file) ){
			
			ob_start();
			require_once $options_file;
			
			$content = ob_get_contents();
			ob_clean();
 
			if( trim($content) != "" ){
				$options = json_decode( $content, true );
			}
		}
		return $options;
	}
}

if ( ! function_exists( 'WZC_mapper' ) ) {
	/**
	 * Shorthand for WooZone_Contextual Mapper.
	 * @since 1.0
	 * @return WZC_Mapper
	 */
	function WZC_mapper() {
		return WooZone_Contextual()->mapper();
	}
}
if ( ! function_exists( 'WZC_settings' ) ) {
	/**
	 * Shorthand for WZC settings.
	 * @since 1.0
	 * @return WZC_Settings
	 */
	function WZC_settings() {
		return WooZone_Contextual()->settings();
	}
}
if ( ! function_exists( 'WZC_path_dir' ) ) {
	/**
	 * Get file/directory path in WooZone_Contextual.
	 *
	 * @param string $name - path name
	 * @param string $file
	 *
	 * @since 1.0
	 * @return string
	 */
	function WZC_path_dir( $name, $file = '' ) {
		return WooZone_Contextual()->path( $name, $file );
	}
}
if ( ! function_exists( 'WZC_upload_dir' ) ) {
	/**
	 * Temporary files upload dir;
	 * @since 1.0
	 * @return string
	 */
	function WZC_upload_dir() {
		return WooZone_Contextual()->uploadDir();
	}
}
if ( ! function_exists( 'WZC_path_dir' ) ) {
	/**
	 * Get file/directory path in WooZone_Contextual.
	 *
	 * @param string $name - path name
	 * @param string $file
	 *
	 * @since 4.2
	 * @return string
	 */
	function WZC_path_dir( $name, $file = '' ) {
		return WooZone_Contextual()->path( $name, $file );
	}
}
if ( ! function_exists( 'WZC_post_param' ) ) {
	/**
	 * Get param value from $_POST if exists.
	 *
	 * @param $param
	 * @param $default
	 *
	 * @since 1.0
	 * @return null|string - null for undefined param.
	 */
	function WZC_post_param( $param, $default = null ) {
		return isset( $_POST[ $param ] ) ? $_POST[ $param ] : $default;
	}
}
if ( ! function_exists( 'WZC_get_param' ) ) {
	/**
	 * Get param value from $_GET if exists.
	 *
	 * @param $param
	 * @param $default
	 *
	 * @since 1.0
	 * @return null|string - null for undefined param.
	 */
	function WZC_get_param( $param, $default = null ) {
		return isset( $_GET[ $param ] ) ? $_GET[ $param ] : $default;
	}
}
if ( ! function_exists( 'WZC_request_param' ) ) {
	/**
	 * Get param value from $_REQUEST if exists.
	 *
	 * @param $param
	 * @param $default
	 *
	 * @since 4.4
	 * @return null|string - null for undefined param.
	 */
	function WZC_request_param( $param, $default = null ) {
		return isset( $_REQUEST[ $param ] ) ? $_REQUEST[ $param ] : $default;
	}
}
if ( ! function_exists( 'WZC_action' ) ) {
	/**
	 * Get WooZone_Contextual special action param.
	 * @since 1.0
	 * @return string|null
	 */
	function WZC_action() {
		$WZC_action = 'keywords';
		if ( isset( $_GET['WZC_action'] ) ) {
			$WZC_action = $_GET['WZC_action'];
		} elseif ( isset( $_POST['WZC_action'] ) ) {
			$WZC_action = $_POST['WZC_action'];
		}

		return $WZC_action;
	}
}

if ( ! function_exists( 'WZC_asset_url' ) ) {
	/**
	 * Get full url for assets.
	 *
	 * @param string $file
	 *
	 * @since 4.2
	 * @return string
	 */
	function WZC_asset_url( $file ) {
		return WooZone_Contextual()->assetUrl( $file );
	}
}

if ( ! function_exists( 'WZC_Get_IP' ) ) {
	function WZC_Get_IP() 
	{
		//Just get the headers if we can or else use the SERVER global
		if ( function_exists( 'apache_request_headers' ) ) {
			$headers = apache_request_headers();
		} else {
			$headers = $_SERVER;
		}
		//Get the forwarded IP if it exists
		if ( array_key_exists( 'X-Forwarded-For', $headers ) && filter_var( $headers['X-Forwarded-For'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 ) ) {
			$the_ip = $headers['X-Forwarded-For'];
		} elseif ( array_key_exists( 'HTTP_X_FORWARDED_FOR', $headers ) && filter_var( $headers['HTTP_X_FORWARDED_FOR'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 )
		) {
			$the_ip = $headers['HTTP_X_FORWARDED_FOR'];
		} else {
			$the_ip = filter_var( $_SERVER['REMOTE_ADDR'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 );
		}
		return $the_ip;
	}
}

/**
 * Plugin name for WZC_time_elapsed_string.
 *
 * @since 1.0
 * @return string
 */
if ( ! function_exists( 'WZC_time_elapsed_string' ) ) {
	function WZC_time_elapsed_string($ptime)
	{
	    $etime = time() - $ptime;
	
	    if ($etime < 1)
	    {
	        return '0 seconds';
	    }
	
	    $a = array( 365 * 24 * 60 * 60  =>  'year',
	                 30 * 24 * 60 * 60  =>  'month',
	                      24 * 60 * 60  =>  'day',
	                           60 * 60  =>  'hour',
	                                60  =>  'minute',
	                                 1  =>  'second'
	                );
	    $a_plural = array( 'year'   => 'years',
	                       'month'  => 'months',
	                       'day'    => 'days',
	                       'hour'   => 'hours',
	                       'minute' => 'minutes',
	                       'second' => 'seconds'
	                );
	
	    foreach ($a as $secs => $str)
	    {
	        $d = $etime / $secs;
	        if ($d >= 1)
	        {
	            $r = round($d);
	            return $r . ' ' . ($r > 1 ? $a_plural[$str] : $str) . ' ago';
	        }
	    }
	}
}

/**
 * DEBUG wpdb
 */
if ( ! function_exists( 'WZC_debug_query' ) ) {
        // DEBUG
        function WZC_debug_query( $result, $data )
        {
            global $current_user;
            get_currentuserinfo();
        
            if ( current_user_can( 'manage_options' ) )
            {
                global $wpdb, $blog_id;
                1 !== $blog_id
                    AND ! defined( 'DIEONDBERROR' )
                        AND define( 'DIEONDBERROR', true );
        
                $wpdb->show_errors     = true;
                $wpdb->suppress_errors = false;
        
                $output = '<pre style="white-space:pre-line;">';
                    $output .= 'Last Error: ';
                    $output .= var_export( $wpdb->last_error, true );
        
                    $output .= "\n\nLast Query: ";
                    $output .= var_export( $wpdb->last_query, true );
        
                    if ( false === $result )
                    {
                        $result = new WP_Error( 'query_failed', 'No update.', $data );
                    }
                    elseif ( 0 === $result )
                    {
                        $result = new WP_Error( 'update_failed', 'Updated zero rows.', $data );
                    }
                    elseif ( 0 < $result )
                    {
                        $result = 'Success';
                    }
                $output .= '</pre>';
        
                // Only abort, if we got an error
                is_wp_error( $result ) 
                    AND exit( $output.$result->get_error_message() );
            }
        }
}

if ( ! function_exists( 'WZC_db_custom_insert' ) ) {
        function WZC_db_custom_insert($table, $fields, $ignore=false, $wp_way=false) {
            global $wpdb;

            if ( $wp_way && !$ignore ) {
                $r = $wpdb->insert( 
                    $table, 
                    $fields['values'], 
                    $fields['format']
                );
            } else {
            
                $formatVals = implode(', ', array_map('WZC_prepareForInList', $fields['format']));
                $theVals = array();
                foreach ( $fields['values'] as $k => $v ) $theVals[] = $k;

                $q = "INSERT " . ($ignore ? "IGNORE" : "") . " INTO $table (" . implode(', ', $theVals) . ") VALUES (" . $formatVals . ");";
                foreach ($fields['values'] as $kk => $vv)
                    $fields['values']["$kk"] = esc_sql($vv);
  
                $q = vsprintf($q, $fields['values']);
                $r = $wpdb->query( $q );
            }
            return $r;
        }
}

if ( ! function_exists( 'WZC_prepareForInList' ) ) {
        function WZC_prepareForInList($v) {
            return "'".$v."'";
        }
}

/**
 * Plugin name for WooZone_Contextual.
 *
 * @since 1.0
 * @return string
 */
function WZC_plugin_name() {
	return WooZone_Contextual()->pluginName();
}


if ( ! function_exists( 'WZC_country2key' ) ) {
        function WZC_country2key($country) {
        	$key = $country;
            if ( 'co.uk' == $country ) {
            	$key = 'uk';
            }
			else if ( 'co.jp' == $country ) {
				$key = 'jp';
			}
			else if ( 'com.mx' == $country ) {
				$key = 'mx';
			}
			else if ( 'com.br' == $country ) {
				$key = 'br';
			}
			return $key;
        }
}

if ( ! function_exists( 'WZC_escapeJsonString' ) ) {
	function WZC_escapeJsonString($value) {
		// list from www.json.org: (\b backspace, \f formfeed)
	    $escapers = array("\\", "/", "\"", "\n", "\r", "\t", "\x08", "\x0c");
	    $replacements = array("\\\\", "\\/", "\\\"", "\\n", "\\r", "\\t", "\\f", "\\b");
	    $result = str_replace($escapers, $replacements, $value);
	    return $result;
	}
}