<?php
/*
 * Define class WZC_Settings
 * Make sure you skip down to the end of this file, as there are a few
 * lines of code that are very important.
 */
!defined('ABSPATH') and exit;

if (class_exists('WZC_Settings') != true) {
    
    class WZC_Settings
    {
        /*
         * Some required plugin information
         */
        const VERSION = '1.0';
		
		private $save_status = false;
        
        /*
         * Required __construct() function that initalizes the WZC_Settings
         */
        public function __construct()
        {
        	$this->save_settings();
        }
		
		private function save_settings()
		{
			$form_saved = isset( $_REQUEST['WZC-form-saved'] ) ? (boolean) $_REQUEST['WZC-form-saved'] : false;
			$box_id = isset( $_REQUEST['box_id'] ) ? (string) $_REQUEST['box_id'] : '';
			if( $form_saved === true && trim($box_id) != "" ){
				$params = isset( $_REQUEST[$box_id] ) ? $_REQUEST[$box_id] : array();
				
				if( count($params) > 0 ){
					update_option( $box_id, $params );
					$this->save_status = true;
				} 
			}
		}
        
        /*
         * Build options, method
         * ---------------------
         * this will create you interface via options array elements
         */
        public function build_options($options = array(), $box_id = '')
        {
            // reset as array, this will stock all the html content, and at the end return it
            $html = array();
  
            if (count($options) == 0) {
                return 'Please fill with some options content first!';
            }

            $noRowElements = array(
                'message',
                'html',
                'app'
            );
            $settings      = array();
            
            // get the values from DB
            $settings = get_option( $box_id );
			
            $html[] = '<form method="POST" class="WZC-form" id="' . ($box_id) . '" action="">';
            
			$html[] = '<div class="WZC-message WZC-success ' . ( $this->save_status == false ? 'WZC-display-none' : '' ) . '">Saved successfully!</div>';
            // create a hidden input for sending the prefix
            $html[] = 	'<input type="hidden" name="box_id" value="' . ($box_id) . '" />';
			$html[] = 	'<input type="hidden" name="WZC-form-saved" value="1" />';
            
            $box = isset($options) ? $options : array();
            $html[] = $this->tabsHeader($box); // tabs html header
 
            // loop the box elements
            if (isset($options['elements']) && count($options['elements']) > 0) {
                
                // loop the box elements now
                foreach ($options['elements'] as $elm_id => $value) {
                    
                    // some helpers. Reset an each loop, prevent collision
                    $val          = '';
                    $select_value = '';
                    $checked      = '';
                    $option_name  = isset($option_name) ? $option_name : '';
                    
                    // Set default value to $val
                    if (isset($value['std'])) {
                        $val = $value['std'];
                    }
                   
                    // If the option is already saved, ovveride $val
                    if ( ( $value['type'] != 'info' ) ) {
                        if ( isset($settings[($elm_id)] )
                            && (
                                ( !is_array($settings[($elm_id)]) && @trim($settings[($elm_id)]) != "" )
                                ||
                                ( is_array($settings[($elm_id)]) /*&& !empty($settings[($elm_id)])*/ )
                            )
                        ) {
                                $val = $settings[( $elm_id )];

                                // Striping slashes of non-array options
                                if ( !is_array($val) ) {
                                    $val = stripslashes( $val );
                                    //if($val == '') $val = true;
                                }
                        }
                    }
                    
                    // If there is a description save it for labels
                    $explain_value = '';
                    if (isset($value['desc'])) {
                        $explain_value = $value['desc'];
                    }
                    
                    if (!in_array($value['type'], $noRowElements)) {
                        // the row and the label 
                        $html[] = '<div class="WZC-form-row' . ($this->tabsElements($box, $elm_id)) . '">
							   <label for="' . ($elm_id) . '">' . (isset($value['title']) ? $value['title'] : '') . '</label>
							   <div class="WZC-form-item">';
                    }
                    
                    switch ($value['type']) {
                        
                        // Basic text input
                        case 'text':
                            $html[] = '<input ' . (isset($value['force_width']) ? "style='width:" . ($value['force_width']) . "px;'" : '') . ' id="' . esc_attr($elm_id) . '" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . ']" type="text" value="' . esc_attr($val) . '" />';
                            break;
							
						// Wordpress color picker input
						case 'color_picker':
							$html[] = '<input class="WZC-wp-color-picker" ' . ( isset($value['force_width']) ? "style='width:" . ( $value['force_width'] ) . "px;'" : '' ) . ' id="' . esc_attr( $elm_id ) . '" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . ']" type="text" value="' . esc_attr( $val ) . '" />';
						break;
                        
                        // Basic checkbox input
                        case 'checkbox':
                            $html[] = '<input ' . (isset($value['force_width']) ? "style='width:" . ($value['force_width']) . "px;'" : '') . ' ' . ($val == true ? 'checked' : '') . ' id="' . esc_attr($elm_id) . '" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . ']" type="checkbox" value="" />';
                            break;
                        
                        // Basic upload_image
                        case 'upload_image':
                            $html[] = '<table border="0">';
                            $html[] = '<tr>';
                            $html[] = '<td>';
                            $html[] = '<input class="upload-input-text" name="' . ($elm_id) . '" id="' . ($elm_id) . '_upload" type="text" value="' . ($val) . '" />';
                            
                            $html[] = '<script type="text/javascript">
								jQuery("#' . ($elm_id) . '_upload").data({
									"w": ' . ($value['thumbSize']['w']) . ',
									"h": ' . ($value['thumbSize']['h']) . ',
									"zc": ' . ($value['thumbSize']['zc']) . '
								});
							</script>';
                            
                            $html[] = '</td>';
                            $html[] = '<td>';
                            $html[] = '<a href="#" class="button upload_button" id="' . ($elm_id) . '">' . ($value['value']) . '</a> ';
                            $html[] = '<a href="#" class="button reset_button ' . $hide . '" id="reset_' . ($elm_id) . '" title="' . ($elm_id) . '">Remove</a> ';
                            $html[] = '</td>';
                            $html[] = '</tr>';
                            $html[] = '</table>';
                            
                            $html[] = '<a class="thickbox" id="uploaded_image_' . ($elm_id) . '" href="' . ($val) . '" target="_blank">';
                            
                            if (!empty($val)) {
                                $imgSrc = $WZC->image_resize($val, $value['thumbSize']['w'], $value['thumbSize']['h'], $value['thumbSize']['zc']);
                                $html[] = '<img style="border: 1px solid #dadada;" id="image_' . ($elm_id) . '" src="' . ($imgSrc) . '" />';
                            }
                            $html[] = '</a>';
                            
                            break;
                        
                        // Basic textarea
                        case 'textarea':
                            $cols = "120";
                            if (isset($value['cols'])) {
                                $cols = $value['cols'];
                            }
                            $height = "style='height:120px;'";
                            if (isset($value['height'])) {
                                $height = "style='height:{$value['height']};'";
                            }
                            
                            $html[] = '<textarea id="' . esc_attr($elm_id) . '" ' . $height . ' cols="' . ($cols) . '" name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . ']">' . esc_attr($val) . '</textarea>';
                            break;
                        
                        // Basic html/text message
                        case 'message':
                            $html[] = '<div class="WZC-message WZC-' . ($value['status']) . ' ' . ($this->tabsElements($box, $elm_id)) . '">' . ($value['html']) . '</div>';
                            break;
                        
                        
                        // Basic html/text message
                        case 'html':
                            $html[] = str_replace("__box_id__", $box_id, $value['html']);
                            break;
                        
                        // Select Box
                        case 'select':
                            $html[] = '<select ' . (isset($value['force_width']) ? "style='width:" . ($value['force_width']) . "px;'" : '') . ' name="' . ( $box_id ) . '[' . esc_attr( $elm_id ) . ']" id="' . esc_attr($elm_id) . '">';
                            
                            foreach ($value['options'] as $key => $option) {
                                $selected = '';
                                if ($val != '') {
                                    if ($val == $key) {
                                        $selected = ' selected="selected"';
                                    }
                                }
                                $html[] = '<option' . $selected . ' value="' . esc_attr($key) . '">' . esc_html($option) . '</option>';
                            }
                            $html[] = '</select>';
                            break;
                        
                        // multiselect Box
                        case 'multiselect':
                            $html[] = '<select multiple="multiple" size="3" name="[' . ( $box_id ) . '][' . esc_attr( $elm_id ) . '][]" id="' . esc_attr($elm_id) . '">';
                            
                            if (count($option) > 1) {
                                foreach ($value['options'] as $key => $option) {
                                    $selected = '';
                                    if ($val != '') {
                                        if (in_array($key, $val)) {
                                            $selected = ' selected="selected"';
                                        }
                                    }
                                    $html[] = '<option' . $selected . ' value="' . esc_attr($key) . '">' . esc_html($option) . '</option>';
                                }
                            }
                            $html[] = '</select>';
                            break;
                            
                        // multiselect Box
                        case 'multiselect_left2right':

                            $available = array(); $selected = array();
                            foreach ($value['options'] as $key => $option ) {
                                if( $val != '' ) {
                                    if ( in_array($key, $val) ) { $selected[] = $key; } 
                                }
                            }
                            $available = array_diff(array_keys($value['options']), $selected);
                            
                            $html[] = '<div class="WZC-multiselect-half WZC-multiselect-available" style="margin-right: 2%;">';
                            if( isset($value['info']['left']) ){
                                $html[] = '<h5>' . ( $value['info']['left'] ) . '</h5>';
                            }
                            $html[] = '<select multiple="multiple" size="' . (isset($value['rows_visible']) ? $value['rows_visible'] : 5) . '" name="[' . ( $box_id ) . '][' . esc_attr( $elm_id ) . '-available][]" id="' . esc_attr( $elm_id ) . '-available" class="multisel_l2r_available">';
                            
                            if(count($available) > 0){
                                foreach ($value['options'] as $key => $option ) {
                                    if ( !in_array($key, $available) ) continue 1;
                                    $html[] = '<option value="' . esc_attr( $key ) . '">' . esc_html( $option ) . '</option>';
                                } 
                            }
                            $html[] = '</select>';
                            
                            $html[] = '</div>';
                            
                            $html[] = '<div class="WZC-multiselect-half WZC-multiselect-selected">';
                            if( isset($value['info']['right']) ){
                                $html[] = '<h5>' . ( $value['info']['right'] ) . '</h5>';
                            }
                            $html[] = '<select multiple="multiple" size="' . (isset($value['rows_visible']) ? $value['rows_visible'] : 5) . '" name="[' . ( $box_id ) . '][' . esc_attr( $elm_id ) . '[]" id="' . esc_attr( $elm_id ) . '" class="multisel_l2r_selected">';
                            
                            if(count($selected) > 0){
                                foreach ($value['options'] as $key => $option ) {
                                    if ( !in_array($key, $selected) ) continue 1;
                                    $isselected = ' selected="selected"'; 
                                    $html[] = '<option'. $isselected .' value="' . esc_attr( $key ) . '">' . esc_html( $option ) . '</option>';
                                } 
                            }
                            $html[] = '</select>';
                            $html[] = '</div>';
                            $html[] = '<div style="clear:both"></div>';
                            $html[] = '<div class="multisel_l2r_btn" style="">';
                            $html[] = '<span style="display: inline-block; width: 24.1%; text-align: center;"><input id="' . esc_attr( $elm_id ) . '-moveright" type="button" value="Move Right" class="moveright WZC-button gray"></span>';
                            $html[] = '<span style="display: inline-block; width: 24.1%; text-align: center;"><input id="' . esc_attr( $elm_id ) . '-moverightall" type="button" value="Move Right All" class="moverightall WZC-button gray"></span>';
                            $html[] = '<span style="display: inline-block; width: 24.1%; text-align: center;"><input id="' . esc_attr( $elm_id ) . '-moveleft" type="button" value="Move Left" class="moveleft WZC-button gray"></span>';
                            $html[] = '<span style="display: inline-block; width: 24.1%; text-align: center;"><input id="' . esc_attr( $elm_id ) . '-moveleftall" type="button" value="Move Left All" class="moveleftall WZC-button gray"></span>';
                            $html[] = '</div>';
                        break;
                            
                        default:
							wp_die("Invalid filed types!");
                            
                    }

					// the element description
                    if (isset($value['desc'])){
                        $html[] = '<span class="WZC-form-note">' . ($value['desc']) . '</span>';
					}

  					if (!in_array($value['type'], $noRowElements)) {
                        // close: .WZC-form-row
                        $html[] = '</div></div>';
                    }
                }
                
                // WZC-message use for status message, default it's hidden
                $html[] = '<div class="WZC-message" id="WZC-status-box" style="display:none;"></div>';
                
              
                // buttons for saving settings
                $html[] = '<div class="WZC-button-row">
					<input type="submit" value="Save the settings" class="WZC-button green WZC-saveOptions" />
				</div>';
                
                // close: form
                $html[] = '</form>';
                
                // return the $html
                return implode("\n", $html);
            }
        }

        //make Tabs!
        private function tabsHeader($box) {
            $html = array();

            // get tabs
            $__tabs = isset($box['tabs']) ? $box['tabs'] : array();
 
            $__ret = '';
            if (is_array($__tabs) && count($__tabs)>0) {
                $html[] = '<ul class="tabsHeader">';
                $html[] = '<li style="display:none;" id="tabsCurrent" title=""></li>'; //fake li with the current tab value!
                foreach ($__tabs as $tabClass=>$tabElements) {
                    $html[] = '<li><a href="javascript:void(0);" title="'.$tabClass.'">'.$tabElements[0].'</a></li>';
                }
                $html[] = '</ul>';
                $__ret = implode('', $html);
                
            }
            return $__ret;
        }
        
        private function tabsElements($box, $elemKey) {
            // get tabs
            $__tabs = isset($box['tabs']) ? $box['tabs'] : array();

            $__ret = '';
            if (is_array($__tabs) && count($__tabs)>0) {
                foreach ($__tabs as $tabClass=>$tabElements) {

                    $tabElements = $tabElements[1];
                    $tabElements = trim($tabElements);
                    $tabElements = array_map('trim', explode(',', $tabElements));
                    if (in_array($elemKey, $tabElements)) 
                        $__ret .= ($tabClass.' '); //support element on multiple tabs!
                }
            }
            return ' '.trim($__ret).' ';
        }
    }
}