<?php
/**
 * Filters / auto-find module
 * http://www.aa-team.com
 * ======================
 *
 * @package     WooZone Contextual
 * @author      Andrei Dinca, AA-Team
 * @version     1.0
 */
!defined('ABSPATH') and exit;
if (class_exists('WZC_keywords_filters') != true) {
    class WZC_keywords_filters
    {
        const WXR_VERSION = '1.0';
        
        /**
         * helpers storage
         *
         * @var array
         */
        public $config = array();
        public $wpdb = null;
        public $keywords = array();
        
        private $replace_helpers = array();
        
        /**
         * The constructor
         */
        public function __construct($parent = array())
        {
            global $WooZone_Contextual, $wpdb;
            $this->wpdb = $wpdb;
            $this->config['db_name'] = $this->wpdb->prefix . 'zonecontextual';
        }
        
        public function init()
        {
            global $id, $post;
			
            add_action('the_content', array(
                $this,
                'filter'
            ), 9 );
        
            /*add_action('the_content', array(
                &$this,
                'roll_back_replacements'
            ), 999 );
            
            add_action('get_comment_text', array(
                &$this,
                'filter'
            ));
            
            add_action('the_excerpt', array(
                &$this,
                'filter'
            ), 9 );
            
            add_shortcode( 'WZC', array(
                &$this,
                'shortcode'
            ) );*/
            
            add_action( 'wp_enqueue_scripts', array( 
                &$this, 'add_styles' 
            ), 10);
        }
        
        public function get_all_keywords()
        {
            $this->keywords = array();
            $results = $this->wpdb->get_results( "SELECT * FROM " . ( $this->config['db_name'] ) . " WHERE 1=1 ORDER BY words_count DESC, keyword ASC;", ARRAY_A );
            
            if( count($results) > 0 ){
                foreach ($results as $row) {
                    $this->keywords[$row['id']] = $row['keyword'];
                }
            }
        }
        
        public function keywords_details( $keywords=array(), $ret='json' )
        {
            $details = array();
            $results = $this->wpdb->get_results( "SELECT keyword, max_replacements as mr, product_ASIN as a, product_Title as t, product_DetailPageURL as d, product_SmallImage as s, product_ListPrice as l, product_OfferSummary as o, product_Binding as b FROM " . ( $this->config['db_name'] ) . " WHERE 1=1 AND keyword IN (" . ( implode( ",", $keywords ) ) . ") ORDER BY words_count DESC, char_length(keyword) DESC, keyword ASC", ARRAY_A );
            if( count($results) > 0 ){
                foreach ($results as $row) {
                    $details[$row['keyword']][$row['a']] = $row;
                }
            }
            
            if( $ret == 'json' ) return json_encode( $details );
            
            return $details;
        }
        
        private function reorder_by_keys()
        {
            $__ = array();
            
            foreach ($this->keywords as $key => $keyword) {
                $__[$keyword][] = $key;
            }
            
            $this->keywords = $__;  
        }
        
        public function replace_callback( $matches ) 
        {
            $hash = md5($matches[0]);
            $this->replace_helpers['replacements'][$hash] = $matches[0];
            return "[WZC ids=\"" . ( implode( ", ", $this->replace_helpers['ids'] ) ) . "\"]" . ( $hash ) . "[/WZC]";
        }
        
        public function roll_back_replacements( $content )
        {
            if( count($this->replace_helpers['ignores_founds']) > 0 ) {
                foreach ($this->replace_helpers['ignores_founds'] as $found_key => $found_val ) {
                    $content = str_replace( $found_key, $found_val, $content );
                }
            }
            
            if( !isset($this->replace_helpers['replacements']) ) return $content;
             
            foreach ( $this->replace_helpers['replacements'] as $key => $keyword ) {
                $content = str_replace( $key, $keyword, $content);
                $this->wpdb->query( $this->wpdb->prepare( "UPDATE `" . ( $this->config['db_name'] ) . "` SET hits=hits+1 WHERE keyword='%s'", $keyword ) );
            }
            
            return $content;
        }
        
        public function ignore_html_parts( $content='' )
        {
            require_once WooZone_Contextual()->path( 'HELPERS_DIR', 'phpQuery.php' );
            $doc = phpQuery::newDocument( $content );
            $ignores_founds = array();
            
            // replace the shortcodes
            preg_match_all("/\[.*?\]/s", $content, $shortcode_matches );
            if( count($shortcode_matches[0]) > 0 ){
                 
                foreach ( $shortcode_matches[0] as $shortcode ) {
                    $content = str_replace( 
                        $shortcode, 
                        md5($shortcode), 
                        $content 
                    );
                    
                    $ignores_founds[ md5($shortcode) ] = $shortcode;
                }
            }
            
            // replace all a html tags
            preg_match_all('/<a .*?<\/a>/i', $content, $a_matches );
            if( count($a_matches) > 0 & isset($a_matches[0]) ){
                foreach ($a_matches[0] as $html_elm) {
                    
                    $content = str_replace( 
                        $html_elm, 
                        md5($html_elm), 
                        $content 
                    );
                    
                    $ignores_founds[ md5($html_elm) ] = $html_elm;
                }
            }  
            
            // replace all img html tags
            preg_match_all("/<img [\w\W]+?>/is", $content, $img_matches );
            if( count($img_matches) > 0 & isset($img_matches[0]) ){
                foreach ($img_matches[0] as $html_elm) {
                    
                    $content = str_replace( 
                        $html_elm, 
                        md5($html_elm), 
                        $content 
                    );
                    
                    $ignores_founds[ md5($html_elm) ] = $html_elm;
                }
            }   
            
            $this->replace_helpers['ignores_founds'] = $ignores_founds;
             
            return $content;
        }
        
        public function filter( $content, $iscomment=0 )
        {
            global $id, $post;
			
            if( is_single() == false ) return $content;
			
            $country = WZC_Amazon_Helper()->get_country_perip_external();
			
            // get all keywords
            $this->get_all_keywords();
			
            // return content if no keywords
            if( count($this->keywords) == 0 ) return $content;
            
            $this->reorder_by_keys();

            $found_keywords = array();
            foreach ($this->keywords as $keyword => $ids) {
                $keyword = preg_quote( $keyword, '/' );
                if( preg_match("/(\b" . ( $keyword ) . "\b)/i", $content) ){
                    $found_keywords[] = "'" . $keyword . "'";
                }
            }
            if( count($found_keywords) == 0 ) return $content;
            $found_keywords = array_unique( $found_keywords );

            // sort combinations first based on depth (DESC) and second on words (ASC) 
            //uasort($found_keywords, array($this, 'compare_founds_depth__'));
            
            $config = WZC_Amazon_Helper()->config;

            $new_content  = "<div class='WZC-the-content'>";
            //$new_content  = "<div class='WZC-the-content' data-keywords='" . ( $this->keywords_details( $found_keywords ) ) . "'>";
            //$new_content  = "<div class='WZC-the-content' data-link_color='" . ( $link_color ) . "' data-link_target='" . ( $link_target ) . "' data-link_maxreplace='" . ( $link_maxreplace ) . "' data-keywords='" . ( $this->keywords_details( $found_keywords ) ) . "'>";
            $__keywords = $this->keywords_details( $found_keywords, 'plain_text' );
            $__keywords = htmlentities( json_encode( $__keywords ) );
            $new_content .= "<span class='WZC-keywords' style='display:none;'>" . $__keywords . "</span>";
            //$new_content .= "<span style='display:none;' id='WZC-country-details' data-country='" . ( stripslashes( json_encode( $country ) ) ) . "'></span>";
            
            $options = array(
                'link_color'            => $config['link_color'],
                'link_target'           => $config['link_target'],
                'link_maxreplace'       => (int) $config['link_maxreplace'],
                'find_distinct_words'   => $config['find_distinct_words'],
                'tooltip_max_height'    => (int) $config['tooltip_max_height'],
                'tooltip_margin_left'   => (int) $config['tooltip_margin_left'],
                'tooltip_margin_top'    => (int) $config['tooltip_margin_top'],
                'country'               => stripslashes( json_encode( $country ) ),
            );
            $options = htmlentities( json_encode( $options ) );
			
            $new_content .= "<span class='WZC-options' style='display:none;'>" . $options . "</span>";
            //$new_content .= "<span style='display:none;' id='WZC-country-details' data-country='" . ( stripslashes( json_encode( $country ) ) ) . "'></span>";
  
            $new_content .=     $content;
            $new_content .= '</div>';
  
            return do_shortcode( $new_content );
        }
        
        public function shortcode( $atts, $content = "" ) 
        {
            $atts = shortcode_atts( array(
                'ids' => ''
            ), $atts );
            
            
            extract( $atts );
            if( trim($ids) != "" ){
                $ids = explode( ", ", $ids );
                if( count($ids) > 0 ){
                    $keywords_details = $this->keywords_details( $ids );
                    $keyword = $keywords_details[0]['keyword'];
                    
                    $html = array();
                    $url = '';
                    $items = array();
                    foreach ( $keywords_details as $row ) {
                        $url = $row['product_DetailPageURL'];
                        
                        $items[] = array(
                            's' => $row['product_SmallImage'],
                            't' => $row['product_Title'],
                            'b' => $row['product_Brand'],
                            'l' => $row['product_ListPrice'],
                            'o' => $row['product_OfferSummary'],
                            'd' => ($row['product_DetailPageURL'])
                        );
                    }
                     
                    $link_color = WZC_Amazon_Helper()->config['link_color'];
                    $content = str_replace( $content, '<a class="WZC-tooltip" style="color:' . ( $link_color ) . '" data-items=\'' . ( stripslashes( json_encode($items) ) ) . '\' href="' . ( $url ) . '">' .  $content . '</a>', $content );
                }
            }
            
            return $content;
        }
        
        public function add_styles()
        {
            wp_enqueue_style( 'WZC-tooltips', WZC_asset_url( 'frontend.css' ), array(), self::WXR_VERSION );
            wp_enqueue_script( 'WZC-script', WZC_asset_url( 'frontend.js' ), array(), self::WXR_VERSION );
            wp_localize_script( 'WZC-script', 'WZC_ajaxurl', admin_url('admin-ajax.php') );
        }
    
        public function compare_founds_depth__($x, $y) {
            $a = array(
                'depth'     => count( explode(' ', $x) ),
                'words'     => $x,
            );
            $b = array(
                'depth'     => count( explode(' ', $y) ),
                'words'     => $y,
            );

            $retval = (int) ( $a['depth'] < $b['depth'] ? 1 : ( $a['depth'] == $b['depth'] ? 0 : -1 ) );
            if ( !$retval ) {
                $retval = strnatcmp($a['words'], $b['words']);
            }
            return $retval;
        }
    }
}