<?php
/**
 * Import / keywords module
 * http://www.aa-team.com
 * ======================
 *
 * @package		WooZone Contextual
 * @author		Andrei Dinca, AA-Team
 * @version		1.0
 */
!defined('ABSPATH') and exit;
if (class_exists('WZC_keywords') != true) {
    class WZC_keywords
    {
        const WXR_VERSION = '1.0';
        
        /**
         * parent storage
         *
         * @var array
         */
        public $parent = array();
        public $config = array();
		public $wpdb = null;
        public $search_in_post_meta = array();
        
        /**
         * The constructor
         */
        public function __construct($parent = array())
        {
        	global $wpdb;
			$this->wpdb = $wpdb;
			
            // load parent
            $this->parent = $parent;
            
            // create AJAX request
			add_action('wp_ajax_WZC_keywords_list_table', array(
                $this,
                '_ajax_fetch_list_table'
            ));
			
			add_action('wp_ajax_WZC_remove_keyword', array(
                $this,
                'keywords_remove'
            ));
			
			$this->config['db_name'] = $this->wpdb->prefix . 'zonecontextual';
			
			require_once WooZone_Contextual()->path( 'KEYWORDS_DIR', 'list-table.class.php' );
			
			WZC_Amazon_Helper();
        }
		
		private function current_sub_action()
		{
			return isset($_REQUEST['sub_action']) ? $_REQUEST['sub_action'] : '';
		}
        
        public function print_interface()
        {
            $html = array();
			
            if( 'new' == $this->current_sub_action() ) {
            	
            	return $this->form();  
            }

			else if( 'modify' == $this->current_sub_action() ){

				$item_id = isset($_REQUEST['id']) ? (int) $_REQUEST['id'] : 0;
				return $this->form( array(), 'modify', 'save-modify', $item_id, 'Modify Keyword Phrase' );
			}
            else if( in_array($this->current_sub_action(), array('save-new', 'save-modify')) ){
                
                $result = $this->save( $this->current_sub_action() );
                    
                if ( 'save-new' == $this->current_sub_action() ) {
                    if ( 'invalid' == $result['status'] && $result['new_id'] > 0 ) {
                        return $this->form( array( 'action_status' => $result ), 'modify', 'save-modify', $result['new_id'], 'Modify Keyword Phrase' );
                    }
                }

                return $this->list_items( array( 
                    'action_status' => $result,
                ));
            }
            else if( 'delete' == $this->current_sub_action() ){
                
                $item_id = isset($_REQUEST['id']) ? (int) $_REQUEST['id'] : 0;

                $result = $this->keywords_remove( $item_id );
                return $this->list_items( array( 
                    'action_status' => $result,
                ));
            } else {

    			return $this->list_items();
            }
			
           	wp_die("invalid action!");
        }

        public function keywords_remove( $keyword_id=0 )
        {
            $base = array(
                'status'        => 'invalid',
                'text'          => 'Keyword #ID is not defined.',
                'new_id'        => $keyword_id,
            );

            $keyword_id = (int) $keyword_id;
            $is_ajax = false;
            if( $keyword_id <= 0 ){
                $keyword_id = isset($_REQUEST['keyword_id']) ? (int) $_REQUEST['keyword_id'] : 0;
                $is_ajax = true;
            }
            
            $status = 'not-defined';
            if ( $keyword_id > 0 ) {
                $status = $this->wpdb->delete( $this->config['db_name'], array( 'id' => $keyword_id ) );
            }

            if ( $status === 'not-defined' ) {}
            else if ( $status ) {

                $base = array_merge($base, array(
                    'status'        => 'valid',
                    'text'          => sprintf( 'Keyword #ID: <strong>%s</strong> deleted successfully.', $keyword_id ),
                ));
            }
            else {

                $base = array_merge($base, array(
                    'status'        => 'invalid',
                    'text'          => sprintf( 'Keyword #ID: <strong>%s</strong> error on deletion.', $keyword_id ),
                ));
            }

            if ( !$is_ajax ) {
                return $base;
            }
            die( json_encode( $base ) );
        }
		
		public function save( $action, $params=array() )
		{
            if ( empty($params) || !isset($params['phrase']) ) {
                $params = $_REQUEST;
            }
            $params = stripslashes_deep( $params );
            $product = isset($params['product']) ? (array) $params['product'] : (array) $params;

            $base = array(
                'status'        => 'invalid',
                'text'          => 'Keyword #ID is not defined.',
                'new_id'        => 0,
            );
 
            // keyword & product ASIN
            $keyword = isset($params['phrase']) ? $params['phrase'] : ( isset($params['keyword']) ? $params['keyword'] : '' );
            $keyword = trim($keyword);
            if ( empty($keyword) ) {
                $base['text'] = 'Empty keyword error.';
                return $base;
            }
            if( !isset($product['ASIN']) || empty($product['ASIN']) || strlen($product['ASIN']) != 10 ) {
                $base['text'] = "Invalid product ASIN.";
                return $base;
            }
           
            // keywords word count            
            $words_count = $this->keyword_count_words( $keyword );
            
            // check if already exists
            $item_exists = $this->wpdb->get_row( $this->wpdb->prepare( "SELECT * FROM " . ( $this->config['db_name'] ) . " WHERE keyword = %s AND product_ASIN=%s", $keyword, $product['ASIN'] ), ARRAY_A );
            if( (boolean) $item_exists == true ){
                $base['new_id'] = (int) $item_exists['id'];
                $base['text'] = sprintf( 'Pair (Keyword: %s, product ASIN: %s) already exists with #ID: %s', $keyword, $product['ASIN'], $item_exists['id'] );
            }
            
            $do_update = false;
            if ( $action == 'save-new' ) {
                if( (boolean) $item_exists == true ){
                    return $base;
                }
            } else if ( $action == 'save-modify' ) {
                if( (boolean) $item_exists == true ){
                    $do_update = true;
                }
            }
            
            $max_replacements = isset($params['max_replacements']) ? (string) $params['max_replacements'] : '';
            $__data = array(
                'keyword'                       => $keyword,
                'words_count'                   => $words_count,
                'max_replacements'              => $max_replacements,
                'product_ASIN'                  => isset($product['ASIN']) ? $product['ASIN'] : '',
                'product_Title'                 => isset($product['Title']) ? $product['Title'] : '',
                'product_Brand'                 => isset($product['Brand']) ? $product['Brand'] : '',
                'product_DetailPageURL'         => isset($product['DetailPageURL']) ? urldecode($product['DetailPageURL']) : '',
                'product_SmallImage'            => isset($product['SmallImage']) ? $product['SmallImage'] : '',
                'product_Binding'               => isset($product['Binding']) ? $product['Binding'] : '',
                'product_ListPrice'             => isset($product['ListPrice']) ? $product['ListPrice'] : '',
                'product_OfferSummary'          => isset($product['OfferSummary']) ? $product['OfferSummary'] : '',
                //'product_EditorialReviews'    => isset($product['EditorialReviews']) ? $product['EditorialReviews'] : '',
            );
            $__data_format = array( 
                '%s',
                '%d',
                '%s',
                '%s', 
                '%s', 
                '%s', 
                '%s', 
                '%s', 
                '%s', 
                '%s', 
                '%s', 
                //'%s',
            );
            
            if ( $do_update ) { // UPDATE
                
                $__where = array(
                    'id'    => $base['new_id'],
                );
                $__where_format = array(
                    '%d',
                );
                
                $dbStat = $this->wpdb->update(
                    $this->config['db_name'], 
                    $__data,
                    $__where,
                    $__data_format,
                    $__where_format
                );

                if ( $dbStat === false ){
                    $base['text'] = sprintf( __LINE__ . '/ Pair (Keyword: %s, product ASIN: %s) unable to be saved. error: %s', $keyword, $product['ASIN'], $this->wpdb->last_error );
                }
                else {
                    $base['text'] = sprintf( 'Pair (Keyword: %s, product ASIN: %s) updated successfully with #ID: %s', $keyword, $product['ASIN'], $base['new_id'] );
                    $base['new_id'] = $base['new_id'];
                    $base['status'] = 'valid';
                }
                $this->update_keyword_max_replacements( $keyword, $max_replacements );
            } else { // INSERT

                /*$dbStat = $this->wpdb->insert(
                    $this->config['db_name'], 
                    $__data,
                    $__data_format
                );*/
                $dbStat = WZC_db_custom_insert(
                    $this->config['db_name'],
                    array(
                        'values' => $__data,
                        'format' => $__data_format
                    )
                );
                 
                if ( (int) $this->wpdb->insert_id <= 0 ){
                    $base['text'] = sprintf( __LINE__ . '/ Pair (Keyword: %s, product ASIN: %s) unable to be saved. error: %s', $keyword, $product['ASIN'], $this->wpdb->last_error );
                }
                else {
                    $base['text'] = sprintf( 'Pair (Keyword: %s, product ASIN: %s) inserted successfully with #ID: %s', $keyword, $product['ASIN'], $this->wpdb->insert_id );
                    $base['new_id'] = $this->wpdb->insert_id;
                    $base['status'] = 'valid';
                    
                    $this->update_keyword_max_replacements( $keyword, $max_replacements );
                }
            }
			return $base;
		}

        private function update_keyword_max_replacements($keyword, $max_replacements) {
            $base = array(
                'status'        => 'invalid',
                'text'          => 'Keyword #ID is not defined.',
            );

            $__data = array(
                'max_replacements'              => $max_replacements,
            );
            $__data_format = array( 
                '%s',
            );
            
            if ( 1 ) { // UPDATE
                
                $__where = array(
                    'keyword'    => $keyword,
                );
                $__where_format = array(
                    '%s',
                );
                
                $dbStat = $this->wpdb->update(
                    $this->config['db_name'], 
                    $__data,
                    $__where,
                    $__data_format,
                    $__where_format
                );
                
                if ( $dbStat === false ){
                    $base['text'] = sprintf( __LINE__ . '/ Keyword: %s - unable to be saved. error: %s.', $keyword, $this->wpdb->last_error );
                }
                else {
                    $base['text'] = sprintf( 'Keyword: %s - updated successfully.', $keyword );
                    $base['status'] = 'valid';
                }
            }
            return $base;
        }

        public function get_keyword_by_id( $id=0 )
        {
            return $this->wpdb->get_row( "SELECT * FROM " . ( $this->config['db_name'] ) . " WHERE id='" . ( $id ) . "'", ARRAY_A );
        }
        
		public function form( $pms=array(), $type='add', $sub_action='save-new', $item_id=0, $form_title='Add New Keyword Phrase' )
		{
            // pms: action_status
            extract($pms);

			$html = array();
			
			if( (int) $item_id > 0 ){
				$form_details = $this->get_keyword_by_id( $item_id );
			}
            
            $stat_msg = '<span class="WZC-action-status WZC-message" style="display: none;"></span>';
            if ( isset($action_status) && !empty($action_status)
                && is_array($action_status) && isset($action_status['status'], $action_status['text']) ) {
                
                switch ($action_status['status']) {
                    case 'invalid':
                        $stat_msg = '<span class="WZC-action-status WZC-message WZC-error">' . $action_status['text'] . '</span>';                        
                        break;
                        
                    case 'valid':
                        $stat_msg = '<span class="WZC-action-status WZC-message WZC-success">' . $action_status['text'] . '</span>';
                        break; 
                }
            }
			
			$html[] = '<div class="wrap">';
			$html[] = 	'<h2>' . ( $form_title ) . '</h2>';
            
            $html[] =   $stat_msg;
            
			$html[] = 	'<form method="POST" class="WZC-form">';
			$html[] = 	'<input type="hidden" name="sub_action" value="' . ( $sub_action ) . '">';
			$html[] = 	'<div class="WZC-form-row WZC-find-amz-product">
							<label for="protocol">Keyword phrase</label>
							<div class="WZC-form-item">
								<input type="text" name="keyword" style="width: 360px;" value="' . ( isset($form_details['keyword']) ? $form_details['keyword'] : '' ) . '" />
								<div id="WZC-dropdown-amazon-selector"></div>
								' . ( WZC_Amazon_Helper()->categs_as_select('node') ) . '
								<a href="#" class="WZC-button blue WZC-small" id="WZC-search-on-amazon">Find product</a>
								<span class="WZC-form-note">The plugin will search on each pages to replace this keywords phrase.</span>
								<input type="text" name="max_replacements" style="width: 120px;" value="' . ( isset($form_details['max_replacements']) ? $form_details['max_replacements'] : '' ) . '" /> (maximum replacements - leave empty to use the global value)
							</div>
						</div>';
						
			$html[] = 	'<div class="WZC-form-row WZC-product-db-details">
							<label for="protocol">Amazon Product ASIN</label>
							<div class="WZC-form-item">
								<input type="text" name="product[ASIN]" style="width: 120px;" value="' . ( isset($form_details['product_ASIN']) ? $form_details['product_ASIN'] : '' ) . '" />
								<a href="#" class="WZC-button blue WZC-small WZC-find-product-by-id">Find product</a>
								<span class="WZC-form-note">The plugin will search on each pages to replace this keywords phrase.</span>
								<table class="WZC-product-details">
									<tr>
										<td>Title:</td>
										<td><input type="text" name="product[Title]" value="' . ( isset($form_details['product_Title']) ? $form_details['product_Title'] : '' ) . '" /></td>
									</tr>
									<tr>
										<td>Brand:</td>
										<td><input type="text" name="product[Brand]" value="' . ( isset($form_details['product_Brand']) ? $form_details['product_Brand'] : '' ) . '" style="width: 200px" /></td>
									</tr>
									<tr>
										<td>Detail Page URL:</td>
										<td><input type="text" name="product[DetailPageURL]" value="' . ( isset($form_details['product_DetailPageURL']) ? $form_details['product_DetailPageURL'] : '' ) . '" /></td>
									</tr>
									<tr>
										<td>Small Image:</td>
										<td><input type="text" name="product[SmallImage]" value="' . ( isset($form_details['product_SmallImage']) ? $form_details['product_SmallImage'] : '' ) . '" /></td>
									</tr>
									<tr>
										<td>Binding:</td>
										<td><input type="text" name="product[Binding]" value="' . ( isset($form_details['product_Binding']) ? $form_details['product_Binding'] : '' ) . '" style="width: 200px" /></td>
									</tr>
									<tr>
										<td>List Price:</td>
										<td><input type="text" name="product[ListPrice]" value="' . ( isset($form_details['product_ListPrice']) ? $form_details['product_ListPrice'] : '' ) . '" style="width: 100px" /></td>
									</tr>
									<tr>
										<td>Lowest New Price:</td>
										<td><input type="text" name="product[OfferSummary]" value="' . ( isset($form_details['product_OfferSummary']) ? $form_details['product_OfferSummary'] : '' ) . '" style="width: 100px" /></td>
									</tr>
									<!--tr>
										<td>Editorial Reviews:</td>
										<td><textarea name="product[EditorialReviews]">' . ( isset($form_details['product_EditorialReviews']) ? $form_details['product_EditorialReviews'] : '' ) . '</textarea></td>
									</tr-->
								</table>
							</div>
						</div>';
						
			$html[] = 	'<div class="WZC-button-row">
							<input type="submit" class="WZC-button green" value="Save Keyword">
						</div>';
						
			$html[] = 	'</form>';
			$html[] = '</div>';
			
            return implode( "\n", $html );
		}


        public function list_items( $pms=array() )
        {
            // pms: action_status
            extract($pms);
            
            // Create an instance of our package class...
            $list_table = new WZC_Keywords_List_Table();
            
            // Fetch, prepare, sort, and filter our data...
            $list_table->prepare_items();
            
            $stat_msg = '<span class="WZC-action-status WZC-message" style="display: none;"></span>';
            if ( isset($action_status) && !empty($action_status)
                && is_array($action_status) && isset($action_status['status'], $action_status['text']) ) {
                
                switch ($action_status['status']) {
                    case 'invalid':
                        $stat_msg = '<span class="WZC-action-status WZC-message WZC-error">' . $action_status['text'] . '</span>';                        
                        break;
                        
                    case 'valid':
                        $stat_msg = '<span class="WZC-action-status WZC-message WZC-success">' . $action_status['text'] . '</span>';
                        break; 
                }
            }
            
            ob_start();
            ?>
            <div class="wrap">
                <h2>Keywords <a class="add-new-h2" href="<?php echo sprintf( '?page=%s&sub_action=%s', $_REQUEST['page'], 'new' );?>">Add New</a></h2>
                
                <?php echo $stat_msg; ?>
                
                <form id="WZC-keywords-list-table" method="get">
                    <input type="hidden" name="page" value="<?php echo $_REQUEST['page'] ?>" />
                    <?php $list_table->display() ?>
                </form>
                
            </div>
            <?php
            $html = ob_get_contents();
            ob_clean();
            return $html;
        }

        public function get_keywords_by_ids( $ids=array() )
        {
            if( count($ids) > 0 ){
                return $this->wpdb->get_results( "SELECT * FROM " . ( $this->config['db_name'] ) . " WHERE 1=1 AND id IN (" . ( implode( ", ", $ids )) . ")", ARRAY_A );
            }
            
            return array();
        }

		public function _ajax_fetch_list_table()
		{
			$list_table = new WZC_Keywords_List_Table();
			$list_table->ajax_response();
		}

        private function print_error($msg = "")
        {
            //!!!TMP
            echo ($msg);
        }
        
        public function keyword_count_words( $keyword ) {
            $words_count = 0;
            if ( !empty($keyword) ) {
                $keyword = trim($keyword);
                $_keyword = explode(' ', $keyword);
                $words_count = count($_keyword);
            }
            return $words_count;
        }
    }
}