<?php
if ( ! class_exists( 'WP_List_Table' ) )
	require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );


/* Hide notices to avoid AJAX errors
 * Sometimes the Class throws a notice about 'hook_suffix' being undefined,
 * which breaks every AJAX call.
 */


class WZC_Keywords_List_Table extends WP_List_Table 
{
	public $data = array();

	private $wpdb = null;
	public $config = array();
	public $_args = array();
	public $_pagination_args = array();
	
	/**
	 * REQUIRED. Set up a constructor that references the parent constructor. We 
	 * use the parent reference to set some default configs.
	 */
	public function __construct() 
	{
		global $status, $page, $wpdb;
	
        $settings = WZC_Amazon_Helper()->config;
    
		// This is used only if making any database queries
		$this->wpdb = $wpdb;
		
		// store database name
		$this->config['db_name'] = $this->wpdb->prefix . 'zonecontextual';
		
		// how many records per page to show
		$this->config['per_page'] = isset($settings['rows_per_page']) ? (int) $settings['rows_per_page'] : 5;
		
		// Set parent defaults
		parent::__construct(
			array(
				// singular name of the listed records
				'singular'	=> 'keyword',
				// plural name of the listed records
				'plural'	=> 'keywords',
				// does this table support ajax?
				'ajax'		=> true,
				'screen'    => 'keyword'
			)
		);
		
		$this->_args['singular'] = 'keyword';
		$this->_pagination_args['order'] = 'DESC';
		$this->_pagination_args['orderby'] = 'id';
	}

	public function column_default( $item, $column_name )
	{
		switch ( $column_name ) {

			case 'hits':
				return "<span class='WZC-statistics-hits'>" . $item[ $column_name ] . "</span>";
			case 'redirects':
				return "<span class='WZC-statistics-redirects'>" . $item[ $column_name ] . "</span>";
			case 'product':
				return '<span class="WZC-product-asin">' . ( $item['product_ASIN'] ) . '</span> ' . ( $item['product_Title'] ) . '';
			case 'date_added':
				return $item[ $column_name ];
			default:
				//Show the whole array for troubleshooting purposes
				return print_r( $item, true );
		}
	}

	public function column_title( $item )
	{
		$page = isset($_REQUEST['page']) ? $_REQUEST['page'] : 'WZC';
        
        $paged = isset($_REQUEST['paged']) ? (int) $_REQUEST['paged'] : 0;
        $paged_var = !empty($paged) ? '&paged='.$paged : '';
        
		// Build row actions
		$actions = array(
			'edit'		=> sprintf( '<a href="?page=%s&sub_action=%s&id=%s%s">Edit</a>', $page, 'modify', $item['id'], $paged_var ),
			'delete'	=> sprintf( '<a href="?page=%s&sub_action=%s&id=%s%s">Delete</a>', $page, 'delete', $item['id'], '' ),
		);
		
		// Return the title contents
		return sprintf('%1$s%2$s',
			/*$1%s*/ $item['keyword'],
			/*$3%s*/ $this->row_actions( $actions )
		);
	}

	public function column_cb( $item )
	{
		 
		return sprintf(
			'<input type="checkbox" name="%1$s[]" value="%2$s" />',
			/*$1%s*/ $this->_args['singular'],  
			/*$2%s*/ $item['id']
		);
	}

	public function get_columns()
	{
		return $columns = array(
			'cb'			=> '<input type="checkbox" />', //Render a checkbox instead of text
			'title'			=> 'Keyword',
			'product'		=> 'Amazon Product',
			'hits'			=> 'Hits',
			//'redirects'		=> 'Redirects',
			'date_added'	=> 'Date'
		);
	}

	public function get_sortable_columns()
	{
		// true means it's already sorted
		return $sortable_columns = array(
			'title'	 		=> array( 'keyword', false ),	
			'hits'			=> array( 'hits', false ),
			'redirects'		=> array( 'redirects', false ),
			'date_added'	=> array( 'date_added', false ),
		);
	}

	public function get_bulk_actions()
	{
		return $actions = array(
			'delete'	=> 'Delete'
		);
	}


	public function process_bulk_action()
	{
		// Detect when a bulk action is being triggered...
		if( 'delete'=== $this->current_action() ) {
			$keyword = isset($_REQUEST['keyword']) ? $_REQUEST['keyword'] : array();
			if( count($keyword) > 0 ) {
				foreach ($keyword as $keyword_id) {
					WZC_keywords()->keywords_remove($keyword_id);  
				}
			}
		}		
	}

	public function prepare_items() 
	{
		$columns = $this->get_columns();
		$hidden = array();
		$sortable = $this->get_sortable_columns();
		
		$this->_column_headers = array( $columns, $hidden, $sortable );
		
		$this->process_bulk_action();
		
		$total_items = $this->wpdb->get_var( "SELECT COUNT(id) FROM " . $this->config['db_name'] );

	    $paged = isset($_REQUEST['paged']) ? max(0, intval($_REQUEST['paged']) - 1) : 0;
	    $orderby = (isset($_REQUEST['orderby']) && in_array($_REQUEST['orderby'], array_keys($this->get_sortable_columns()))) ? $_REQUEST['orderby'] : 'id';
	    $order = (isset($_REQUEST['order']) && in_array($_REQUEST['order'], array('asc', 'desc'))) ? $_REQUEST['order'] : 'desc';
		
		$offset = $paged * $this->config['per_page'];  
	    $this->items = $this->wpdb->get_results($this->wpdb->prepare("SELECT * FROM " . ( $this->config['db_name'] ) . " ORDER BY $orderby $order LIMIT %d OFFSET %d", $this->config['per_page'], $offset), ARRAY_A);

		// REQUIRED for pagination
		$current_page = $this->get_pagenum();
		
		// register our pagination options & calculations.
		$this->set_pagination_args(
			array(
				// WE have to calculate the total number of items
				'total_items'	=> $total_items,
				// WE have to determine how many items to show on a page
				'per_page'	=> $this->config['per_page'],
				// WE have to calculate the total number of pages
				'total_pages'	=> ceil( $total_items / $this->config['per_page'] ),
				// Set ordering values if needed (useful for AJAX)
				'orderby'	=> ! empty( $_REQUEST['orderby'] ) && '' != $_REQUEST['orderby'] ? $_REQUEST['orderby'] : 'id',
				'order'		=> ! empty( $_REQUEST['order'] ) && '' != $_REQUEST['order'] ? $_REQUEST['order'] : 'desc'
			)
		);
	}

	/**
	 * Display the table
	 * Adds a Nonce field and calls parent's display method
	 *
	 * @since 3.1.0
	 * @access public
	 */
	public function display()
	{ 
		wp_nonce_field( 'ajax-custom-list-nonce', '_ajax_custom_list_nonce' );
		
		echo '<input type="hidden" id="ajaxid" name="ajaxid" value="WZC_keywords_list_table" />';
		echo '<input type="hidden" id="order" name="order" value="' . $this->_pagination_args['order'] . '" />';
		echo '<input type="hidden" id="orderby" name="orderby" value="' . $this->_pagination_args['orderby'] . '" />';

		parent::display();
	}

	/**
	 * Handle an incoming ajax request (called from admin-ajax.php)
	 *
	 * @since 3.1.0
	 * @access public
	 */
	public function ajax_response() 
	{
		check_ajax_referer( 'ajax-custom-list-nonce', '_ajax_custom_list_nonce' );

		$this->prepare_items();

		extract( $this->_args );
		extract( $this->_pagination_args, EXTR_SKIP );

		ob_start();
		if ( ! empty( $_REQUEST['no_placeholder'] ) )
			$this->display_rows();
		else
			$this->display_rows_or_placeholder();
		$rows = ob_get_clean();

		ob_start();
		$this->print_column_headers();
		$headers = ob_get_clean();

		ob_start();
		$this->pagination('top');
		$pagination_top = ob_get_clean();

		ob_start();
		$this->pagination('bottom');
		$pagination_bottom = ob_get_clean();

		$response = array( 'rows' => $rows );
		$response['pagination']['top'] = $pagination_top;
		$response['pagination']['bottom'] = $pagination_bottom;
		$response['column_headers'] = $headers;

		if ( isset( $total_items ) )
			$response['total_items_i18n'] = sprintf( _n( '1 item', '%s items', $total_items ), number_format_i18n( $total_items ) );

		if ( isset( $total_pages ) ) {
			$response['total_pages'] = $total_pages;
			$response['total_pages_i18n'] = number_format_i18n( $total_pages );
		}

		die( json_encode( $response ) );
	}
}